package bham.cs.fsc.gp;

import java.util.Observable;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.rmi.*;
import java.net.*;
import java.io.*;
import bham.cs.fsc.ui.*;

/**
* This is the base class for all kinds of function nodes (i.e.
* non-terminals). Each kind of function can have a different number
* of arguments (i.e. subtrees).
*/

public abstract class Function extends Program implements Serializable {

  protected Program[] arg;
  //Program[] arg;

	public String toString(int level) {
		//	Just to please the eye: If all arguments of the function
		//	are terminals, I print a single line instead of spreading the 
		//	stuff tree-like.

	        //System.out.println("??? inside Function.toString(level).");

		boolean allArgsAreTerminals = true;
		for (int a = 0; a < arg.length; a++) {
			allArgsAreTerminals = allArgsAreTerminals && (arg[a] instanceof Terminal);
		}

		//System.out.println("??? allArgsAreTerminals : " + allArgsAreTerminals);
		
		String s = new String();
		if (!allArgsAreTerminals) {
			s = indent(level) + getName() + "(\n";
			int i = 0;
			while (i < arg.length-1) {
			  //System.out.println("??? converting arg " + i + " to string");
				s = s + arg[i].toString(level+1) + ",\n";
				//System.out.println("??? string is : " + s);
				i++;
			}
			if (i < arg.length) {
				s = s + arg[i].toString(level+1) + "\n";
			}
			s = s + indent(level) + ")";
		} else {
			s = indent(level) + getName() + "(";
			int i = 0;
			while (i < arg.length-1) {
			  //System.out.println("??? converting arg " + i + " to string");
				s = s + arg[i].toString(0) + ",";
		//	look at their printed representation:
				//System.out.println("??? string is : " + s);
				i++;
			}
			if (i < arg.length) {
				s = s + arg[i].toString(0);
			}
			s = s + ")";
		}
		return s;
	}

  protected Object clone() {
    //public Object clone() {
		Function temp = null;
		try {
			temp = (Function)getClass().newInstance();
			for (int i = 0; i < arg.length; i++) {
				temp.arg[i] = (Program)arg[i].clone();
			}
		} catch (Exception e) {System.out.println("Exception in clone() of Function.java : " + e); }
		
		return temp;
	}

	int countNodes() {
		int count = 1;
		for (int a = 0; a < arg.length; a++) {
			count = count + arg[a].countNodes();
		}
		return count;
	}

	int countNodes(Condition cond) {
		int count = (cond.test(this) ? 1 : 0);
		for (int a = 0; a < arg.length; a++) {
			count = count + arg[a].countNodes(cond);
		}
		return count;
	}

}




