package bham.cs.fsc.gp;

import java.util.Observable;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Date;
import java.util.zip.*;          // Data compression/decompression streams.
import java.awt.FileDialog;       
import java.rmi.*;
import java.net.*;
import java.io.*;
import bham.cs.fsc.ui.*;
import bham.cs.fsc.exp.*;
import bham.cs.fsc.gp.Individual;
import bham.cs.fsc.gp.Program;
import bham.cs.fsc.gp.Condition;



/**
* This class demonstrates symbolic regression with genetic programming.
* The algorithms are taken from John R. Koza's book
* "Genetic programming: on the programming of computers by means of natural
* selection", Bradford Book, 1993, ISBN 0-262-11170-5.
* <br>
* John Koza holds the copyright for the algorithms.
* My code is a mere translation from Koza's LISP source into Java. I did not
* change most of the variable and function names, so that you will recognize
* the parallels immediately. I even left the wording of the original 
* comments untouched (after all, Koza's English is better than mine).
* <br>
* You find Koza's original code and license agreement <a href="http://www-cs-faculty.stanford.edu/~koza/littlelisp.html">here.</a>  
* <br>
* <br>
* A GP object contains only data and algorithms, but no fancy user interface.
* In the classic Model-View-Controller architecture, you would call it the 
* "model". Whenever the state of the model changes, it sends a message to 
* all its views so that these may update themselves to reflect the change.
* In Java parlance, the views are based on "Observer" and the model
* implements "Observable".
* <br>
* The GP evolution algorithm runs in a thread of its own.
* That's why the class implements interface "Runnable". 
*
* @version 1.0
* @author Original algorithms (c) <a href="http://www-cs-faculty.stanford.edu/~koza/">John R. Koza</a>,<br>
*		Java port by <a href="mailto:gerber@ifh.ee.ethz.ch">Hans U. Gerber</a>
*               Adapted by Phyllis Chong <a href="mailto:fsc@cs.bham.ac.uk"> to be distributed and generic.
*
*/
public class GP extends Observable implements Runnable, Serializable {

	public NumberUI populationSize;
	public NumberUI maxDepthForNewIndividuals;
	public NumberUI maxDepthForIndividualsAfterCrossover;
	public NumberUI maxDepthForNewSubtreesInMutants;
	public NumberUI crossoverFraction;
	public NumberUI fitnessProportionateReproFraction;
	public NumberUI mutationFraction;
	public ChoiceUI methodOfGeneration;
	public ChoiceUI methodOfSelection;
        
        public ChoiceUI elitismChosen;
        //public PointsUI fitnessCases;
	public SetUI terminalSet;
	public SetUI functionSet;

        public NumberUI maxGenNum;
        public NumberUI exportGenNum;
        public NumberUI seed;
        static public NumberUI imigrantQueueSize;
        public NumberUI numToExport;
        public NumberUI tournamentSize;

        //double sumOfAdjustedFitnesses;

	Thread thread = null;
	Individual[] population;
	public int currentGeneration = 0;
        
        Individual bestOfRunIndividual = null;
        int generationOfBestOfRunIndividual = 0;
        //static final int MAX_GENERATIONS = 1000;

        //static int MAX_GENERATIONS;
        //static int imigrantLimit;
        //static int SEED;
        //public static Random random;
        int SEED;
        static public Random random;

        //String fitnessCasesString;
        
        //static final int exportCount = 20;
        //static int exportGen;
        //static final int imigrantLimit = 5;

        //int counter = 1;
        int counter;
        int nodeCount;

        //Individual solution = null;
        Individual solution;
        boolean solutionFound;
        int genNrOfSolution;
        int individualNrOfSolution;
        //static int solutionFirstFoundGenNr;
  //***boolean toSetGenOfSolution;
        //static boolean sendSol;

        private Date currentTime;

        //static Vector solGenNrVector = new Vector();
  //***public static Vector solGenNrVector;
        //public static Hashtable solGenNrTable;

        //boolean bestChg = false;

        String thisHost = "" ;
        //PrintStream log;
        
        Problem problem;

        // static Vector imigrantVector = new Vector(imigrantLimit);
  
        static public Hashtable imigrantTable = new Hashtable();

        //String fileName = "ClientList.dat";
        //static String fileName;
        public String fileName;
        //static String clientFile = new String("/home/students/acs97/fsc/fs-project2/GPsrc/SymRegrs/" + fileName);
        static String clientFile;
        //static final String clientFile = new String("/home/students/acs97/fsc/fs-project2/GPsrc/SymRegrs/ClientList.dat"); 
        DGPClient applet_DGP;

  //*public static int genToStartAnotherPop = 0;
        public static boolean remotePop = false;
       
        File popFile;
        boolean PopFileFound;
        //boolean firstRun = true;
        boolean runComplete = false;
        //boolean firstRun = true; 
  
        public ThreadMonitor m;

        public boolean drawBest = false;
        public boolean evalBest = false;

        boolean NoConnection = true;
        boolean lastSendSucceed = true;
  
        boolean canSendResult = false;

        Individual[] IndArray;
        int selCount = 0;

        public boolean suddenEnd = false;

        int drawChoice;
  
        Hashtable parmTable = new Hashtable();
        String clientID;
        String servletURL;
        Individual[] replyObj;
        String errorMsg;
        
        String solName;
        String solOrg;
        String solDesignation;
  
        StringBuffer sb;

        Hashtable runStateTable;
  
  public static final String servletHost = "http://studentweb.cs.bham.ac.uk/servlets/bham.cs.fsc.";  
  //public static final String servletHost = "http://bert:8080.cs.bham.ac.uk/servlets/bham.cs.fsc.";   

        //imigrant i;

	/**
	* Each GP has its own random number generator.
	*/
  
        //static final int SEED = 12345;
        //static Random random = new Random(SEED);
       
        // static final int SEED ;
        //static int SEED;
        //static Random random ;

        //public GP(int s, int expCount) {
        //public GP(Problem prob, int s) {
	public GP(Problem prob, int s, String fileName, DGPClient applet_DGP) {
        //public GP(Problem prob) {
		//
		//	All the parameters that the user can change are wrapped up with
		//	a user interface component. Let's construct these components here.
		//	This will preset their text labels, default values, initial
		//	selections and value ranges.
		//	To use these components, add them to a dialog.
		//

	        
	        SEED = s;
		random = new Random(SEED);
		//exportGen = expCount;

	        problem = prob;
		this.fileName = fileName;
		this.applet_DGP = applet_DGP;
		/*
		if (fileName != "None") {
		  popFile = new File(fileName);
		}
		*/
		/*
		try {
		  thisHost = new String(InetAddress.getLocalHost().getHostName()) ;
		} 
		catch(java.net.UnknownHostException e) {
	  	  System.out.println("Unknown host exception !");
		}
		*/
		/*
		try {
		log = new PrintStream(new FileOutputStream(thisHost + ".GP.log"));
		}
		catch(IOException e) {
		  System.out.println("IOException while creating log at GP()");
		}
		*/
		/*
		 initFlags() is normally only called at start() and invoked at every press of the 'Start' button.
		   It is included here for use when running experiments 
		   with DGP_Exp. When running multiple experimental runs, GP class will be invoked automatically without 
		   user having to press 'Start' button. Thus, the flags have to be initialised here. 
		   */

		initFlags();
		

		maxGenNum = 
		  //new NumberUI("Max. no. of generations in a run", 50, 20, 1000);
		  new NumberUI("Max. no. of generations in a run", 50, 1, 500);
		exportGenNum = 
		  new NumberUI("No. of generations between migrations", 15, 1, 100);
		numToExport = 
		  new NumberUI("Number of individuals to export", 1, 1, 10);
		imigrantQueueSize =
		  //new NumberUI("Size of imigrant queues", 5, 1, 20);
		  new NumberUI("Number of individuals to import", 2, 1, 10);
		tournamentSize = 
		  new NumberUI("Tournament selection group size", 7, 2, 20);
		//seed = 
		//  new NumberUI("Randomiser seed", 12345, 00000, 99999);

		//SEED = seed.intValue();
		//random = new Random(SEED);

		//populationSize =
		//		new NumberUI("Population size", 100, 10, 1000);

		populationSize =
		  //new NumberUI("Population size", 500, 10, 1000);
		  new NumberUI("Population size", 300, 5, 5000);

		maxDepthForNewIndividuals =
				new NumberUI("Max. depth for new individuals", 6, 1, 10);
		maxDepthForIndividualsAfterCrossover =
		  //  new NumberUI("Max. depth for individuals after crossover", 20, 5, 20);
		  new NumberUI("Max. depth for individuals after crossover", 17, 5, 20);
		// The max dept for mutants is not important as mutation will not be used here according to Koza's bk.
		maxDepthForNewSubtreesInMutants = 
				new NumberUI("Max. depth for new subtrees in mutants", 4, 1, 7);
		crossoverFraction =
		  //  new NumberUI("Crossover percentage", 80, 1, 100);
		  new NumberUI("Crossover percentage", 90, 1, 100);
		fitnessProportionateReproFraction =
		  //new NumberUI("Reproduction percentage", 0, 0, 100);
		  new NumberUI("Reproduction percentage", 10, 0, 100);
		mutationFraction =
		  //		new NumberUI("Mutation percentage", 20, 0, 100);
		  new NumberUI("Mutation percentage", 0, 0, 100);

		problem.makeDefaultFitnessCases();
		//fitnessCasesString = problem.makeDefaultFitnessCases();
		//fitnessCases = new PointsUI("Fitness cases", fitnessCasesString);
		//fitnessCases = new PointsUI("Fitness cases", problem.makeDefaultFitnessCases());
		//fitnessCases = new PointsUI("Fitness cases", makeDefaultFitnessCases());

		/*
		ProgramChoice[] terminalChoices = {
			new ProgramChoice("Random constant", new RandomConstant().getClass()),
			new ProgramChoice("Variable x", new Variable().getClass())
		};
		int[] terminalSelections = {0, 1};
		terminalSet = new SetUI("Terminal set",	terminalChoices, terminalSelections);
		*/

		terminalSet = problem.getTerminalSet();
		
		/*
		ProgramChoice[] functionChoices = {
			new ProgramChoice("+ (add)", new Addition().getClass()),
			new ProgramChoice("- (sub)", new Subtraction().getClass()),
			new ProgramChoice("* (mul)", new Multiplication().getClass()),
			new ProgramChoice("/ (div)", new Division().getClass()),
			new ProgramChoice("sin", new Sine().getClass()),
			new ProgramChoice("cos", new Cosine().getClass()),
			new ProgramChoice("exp", new Exp().getClass())
		};
		int[] functionSelections = {0, 1, 2};
		functionSet = new SetUI("Function set", functionChoices, functionSelections);
		*/

		functionSet = problem.getFunctionSet();

		MethodOfGeneration[] generationChoices = {
			new MethodOfGeneration("Grow", MethodOfGeneration.GROW),
			new MethodOfGeneration("Full", MethodOfGeneration.FULL),
			new MethodOfGeneration("Ramped half and half", MethodOfGeneration.RAMPED_HALF_AND_HALF)
		};
		methodOfGeneration = new ChoiceUI("Method of generation",	generationChoices, 2);

		MethodOfSelection[] selectionChoices = {
			new MethodOfSelection("Fitness proportionate", MethodOfSelection.FITNESS_PROPORTIONATE),
			new MethodOfSelection("Tournament", MethodOfSelection.TOURNAMENT)
		};
		//methodOfSelection = new ChoiceUI("Method of selection",	selectionChoices, 1);
		methodOfSelection = new ChoiceUI("Method of selection",	selectionChoices, 0);

		//SEED = seed.intValue();
		//random = new Random(SEED);
		//MAX_GENERATIONS = maxGenNum.intValue();
		//imigrantLimit = imigrantQueueSize.intValue();

		ElitismSelection[] elitismChoices = {
			new ElitismSelection("Yes", ElitismSelection.ELITISM_USED),
			new ElitismSelection("No", ElitismSelection.ELITISM_NOT_USED)
		};

		elitismChosen = new ChoiceUI("Elitism", elitismChoices, 1);

	}

        // Initialise flags and control variables at every press of START button 
        void initFlags() {  
	     counter = 1;
	     solution = null;
	     //***toSetGenOfSolution = false;
	     //***solGenNrVector = new Vector();
	     //solGenNrTable = new Hashtable();
	     problem.initMaxHits();
	     evalBest = false;
	     drawBest = false;
	     lastSendSucceed = true;
	     selCount = 0;
	     suddenEnd = false;
	     //newly added for applet
	     //boolean PopFileFound;
             //boolean firstRun = true;
             //boolean runComplete = true;

	}

	/**
	* Should be called after any of the settings have changed, will notify
	* any observers of the changes.
	*/
	public void settingsChanged() {
	  //notifyObservers(new GPMessageFitnessCasesSet(problem.fitnessCases.get()));
	  notifyObservers(new GPMessageFitnessCasesSet(problem.getFitnessCases().get()));
	}

	/**
	* After the construction of a GP object, you should call the init() method;
	* this will notify any observers that the initial settings are now
	* established. Don't do it in the constructor, because Observer-Observable
	* relationships are yet not established during construction.
	*/
	public void init() {
		settingsChanged();
	}

	/**
	* Starts the GP algorithm running in its own thread.
	* The priority is set lower than normal, so that a user interface thread
	* with normal priority can surely preempt the GP thread.
	*/
        public synchronized void start() {
        //public synchronized void start(int pSeed) {

	        //SEED = pSeed;
	        //random = new Random(SEED);
	        //random.setSeed(SEED);

	        initFlags();

		thread = new Thread(this);
		thread.setPriority(Thread.NORM_PRIORITY - 1);
		thread.start();
		state = STARTED;
		notifyObservers(new GPMessageStateChanged(state, ""));
	}

	/**
	* Suspends the GP thread.
	*/
	public synchronized void suspend() {
		if (thread != null && thread.isAlive()) {
			state = SUSPENDED;
			notifyObservers(new GPMessageStateChanged(state, ""));
			thread.suspend();
		}
	}

	/**
	* Resumes a suspended GP thread.
	*/
	public synchronized void resume() {
		if (thread != null && thread.isAlive()) {
			thread.resume();
			state = RESUMED;
			notifyObservers(new GPMessageStateChanged(state, ""));
		}
	}

	/**
	* Stops a running GP thread and kills it.
	*/
	public synchronized void stop() {
		//	Note that the GP thread may call stop() to kill itself.
		//	Therefore, any code following thread.stop() below would
		//	never be executed.

	        System.out.println("***** Processing stop() *****"); 
		if (thread != null && thread.isAlive()) {
		  /*
		  if (SEED != applet_DGP.defaultSEED) {

		    if (! runComplete) {
		    System.out.println("run not Completed ");
		    suddenEnd = true;
		    System.out.println("suddenEnd set to true");
		    
		    //	Sort the population so that the roulette wheel selection is easy:
		    System.out.println("Sorting population by fitness in descending order.");
		    sortPopulationByFitness();
		    
		    //if ((canSendResult) && (counter != 1)) { --- ignore checking of counters: difficult to cater for.
		    if (canSendResult) {
		      System.out.println("can send result ... looking for 1 indv and create array to send");
		      //createAndExportIndivArray(numToExport.intValue());
		      createAndExportIndivArray(1);
		    }
		    else 
		      if ((IndArray != null) && (selCount > 0) && (! lastSendSucceed)) {
			System.out.println("IndArray not null and last send not successful. Resending IndArray");
			exportIndividual(IndArray);
		      }
		    }

		    sendEndResults();

		  }
		  */
		  
		  if (SEED != applet_DGP.defaultSEED) {
		    if (! runComplete) {
		      sendEndResults();
		    }
		  }

		  if ((applet_DGP.Application) && (! runComplete)) {
		    savePopulation();
		  }

		  if (evalBest) {
		    suddenEnd = true;
		    System.out.println("Stopped button pressed. evalBest is " + evalBest);
		    System.out.println("setting drawChoice to 2");
		    drawChoice = applet_DGP.drawChoice.getSelectedIndex();
		    applet_DGP.drawChoice.select(2);
		    if (((ElitismSelection)elitismChosen.getSelection()).getValue() == ElitismSelection.ELITISM_USED) {
		      problem.evalBestOfRunIndividual(bestOfRunIndividual,this);
		    }
		    else {
		      problem.evalBestOfRunIndividual(population[0],this);
		    }
		    applet_DGP.drawChoice.select(drawChoice);
		  }
		  state = STOPPED;
		  notifyObservers(new GPMessageStateChanged(state, ""));
		  applet_DGP.settingsButton.enable(true); 
		  thread.stop();
		}
	}

	/**
	* Aborts a running GP thread and notifies any observers with a
	* diagnostic message. This method is tyically called from an
	* exception handler if the GP algorithm crashes unexpectedly.
	*/
	public synchronized void crash() {
		if (thread != null && thread.isAlive()) {
			state = STOPPED;
			notifyObservers(new GPMessageStateChanged(state,
					"The genetic algorithm crashed\n" + 
					"in generation " + currentGeneration + ".\n" +
					"Sorry."));
			thread.stop();
		}
	}

	public static final int IDLE = 0;
	public static final int STARTED = 1;
	public static final int SUSPENDED = 2;
	public static final int RESUMED = 3;
	public static final int STOPPED = 4;

	private int state = IDLE;
	private int oldState = IDLE;

	/**
	* @return the execution state of the GP thread
	*/
	public int getState() {
		return state;
	}

	/**
	* freeze() and thaw() are useful if the GP is running in an applet.
	* A well-behaved applet should stop running when the user moves to
	* another page and it should start again when he returns to the
	* (still loaded) applet.
	* freeze() and thaw() may thus be called from the applet's start()
	* and stop() methods to pause the GP while its applet is not visible.
	*/
	public synchronized void freeze() {
		oldState = getState();
		if (thread != null && thread.isAlive() &&
				(oldState == STARTED || oldState == RESUMED) &&
		                (applet_DGP.bkgdChoice.getSelectedIndex() == 1))
		     { // i.e. thread running & user choose not to run in background
			thread.suspend();
		}
	}

	/**
	* freeze() and thaw() are useful if the GP is running in an applet.
	* A well-behaved applet should stop running when the user moves to
	* another page and it should start again when he returns to the
	* (still loaded) applet.
	* freeze() and thaw() may thus be called from the applet's start()
	* and stop() methods to pause the GP while its applet is not visible.
	*/
	public synchronized void thaw() {
		if (thread != null && thread.isAlive() &&
				(oldState == STARTED || oldState == RESUMED)) {
			thread.resume();
		}
	}

	/**
	* @return a terminal, randomly chosen from the terminal set
	*/
	Terminal chooseFromTerminalSet() {
		Terminal choice;
		try {
			int index = random.nextInt(terminalSet.countSelections());
			Class cls = ((ProgramChoice)terminalSet.getSelectedItem(index)).value();
		        choice = (Terminal)(cls.newInstance());
			//System.out.println("Terminal selected : " + ((ProgramChoice)terminalSet.getSelectedItem(index)).toString());  
		} catch(Exception e) {
			choice = null;
		}
		return choice;
	}

	/**
	*	Creates arguments for a function
	*/
	void createArgumentsForFunction(
		Function function,
		int allowableDepth,
		boolean fullP) {
	        //System.out.println("??? inside createArgumentsForFunction.");
	        //System.out.println("??? no. of arguments for function is : " + function.arg.length);
		for (int i = 0; i < function.arg.length; i++) {
		  //System.out.println("??? Creating argument " + i);
			function.arg[i] = createIndividualProgram(
					allowableDepth,
					false,
					fullP);
		}
	}

        void createArgumentsForPrimeFunction(Function function) {
		//int allowableDepth,
		//boolean fullP) {
	        //System.out.println("??? inside createArgumentsForFunction.");
	        //System.out.println("??? no. of arguments for function is : " + function.arg.length);
		for (int i = 0; i < function.arg.length; i++) {
		  //System.out.println("??? Creating argument " + i);
			function.arg[i] = createPrimeIndividualProgram();
		}
	}
	

	/**
	*	Creates a program recursively using functions and terminals from
	* the respective sets.
	* @param allowableDepth the remaining depth of the tree we can create,
	*		when we hit zero we will only select terminals
	* @param topNodeP is true only when we are being called as the top 
	*		node in the tree. This allows us to make sure that we always put
	*		a function at the top	of the tree.  
	* @param fullP indicates whether this individual is to be maximally
	*		bushy or not
	*/
	Program createIndividualProgram(
		int allowableDepth,
		boolean topNodeP,
		boolean fullP) {
		Program p;
		int choice;
		Function function;

		//System.out.println("??? inside createIndividualProgram");
		if (allowableDepth <= 0) {
  		//	We've reached maxdepth, so just pack a terminal.
			p = chooseFromTerminalSet();
		} else {
			if (fullP || topNodeP) {
  			//	We are the top node or are a full tree, so pick only a function.
				choice = random.nextInt(functionSet.countSelections());
				//System.out.println("??? choiceNo of randomly selected function is : " + choice);
				try {
				        ProgramChoice pChoice = (ProgramChoice)functionSet.getSelectedItem(choice);
					//System.out.println("??? Function selected is : " + pChoice.toString());
					//function = (Function)pChoice.value();
					Class cls = pChoice.value();
					//System.out.println("??? cls is " +  cls.getName());
					//Class cls = ((ProgramChoice)functionSet.getSelectedItem(choice)).value();
					//System.out.println("??? pChoice value assigned to cls");
					//Class c = cls.newInstance();
					//System.out.println("new instance of cls assigned to class c");
					//function = (Function)c;
					//System.out.println("function created.");
					function = (Function)cls.newInstance();
					//System.out.println("??? new instance of function created.");
				} catch(Exception e) {
				  //System.out.println("??? Exception encountered when creating instance of function. Error is : " + e);
				  //System.out.println("??? Function is set to null."); 
					function = null;
				}
				//System.out.println("??? Going to createArgumentsForFunction.");
				createArgumentsForFunction(
						function,
						allowableDepth - 1,
						fullP);
				p = function;
			} else {
				//	Choose one from the bag of functions and terminals.
				choice = random.nextInt(terminalSet.countSelections() + functionSet.countSelections());
				if (choice < functionSet.countSelections()) {
					//	We chose a function, so pick it out and go on creating the tree down from here.
					try {
						Class cls = ((ProgramChoice)functionSet.getSelectedItem(choice)).value();
						function = (Function)cls.newInstance();
						//System.out.println("Function selected : " + ((ProgramChoice)functionSet.getSelectedItem(choice)).toString()); 
					} catch(Exception e) {
						function = null;
					}
					createArgumentsForFunction(
							function,
							allowableDepth - 1,
							fullP);
					p = function;
				} else {
					//	We chose an atom, so pick it out.
					p = chooseFromTerminalSet();
				}
			}
		}
		return p;
	}

        Program createPrimeIndividualProgram() {
	  //Creating prime program for LosAltosHills 
		Program primeP = null;
		//int choice;
		Function primeFunction = null;
		Class cls;
		ProgramChoice pChoice = null;
		nodeCount ++;
		try {
		  switch (nodeCount) {
		  case 1 :  
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(3);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 2 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 3 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 4 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 5 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 6 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 7 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 8 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		   case 9 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 10 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 11 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 12 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 13 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;  
		  case 14 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 15 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 16 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(3);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;    
		  case 17 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;      
		  case 18 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;    
		  case 19 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 20 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 21 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 22 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 23 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;
		  case 24 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 25 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(3);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;    
		  case 26 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;      
		  case 27 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;    
		  case 28 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;    
		  case 29 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 30 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 31 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 32 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 33 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 34 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;    
		  case 35 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;  
		  case 36 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 37 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;    
		  case 38 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;    
		  case 39 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;   
		  case 40 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 41 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break; 
		  case 42 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 43 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 44 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 45 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;   
		  case 46 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 47 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 48 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 49 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break;  
		  case 50 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 51 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 52 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 53 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 54 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 55 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 56 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 57 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 58 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 59 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(1);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 60 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 61 :
		    pChoice = (ProgramChoice)functionSet.getSelectedItem(0);
		    cls = pChoice.value();
		    primeFunction = (Function)cls.newInstance();
		    createArgumentsForPrimeFunction(primeFunction);
		    primeP = primeFunction;
		    break;
		  case 62 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 63 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 64 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 65 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  case 66 :
		    pChoice = (ProgramChoice)terminalSet.getSelectedItem(2);
		    cls = pChoice.value();
		    primeP = (Terminal)cls.newInstance();
		    break; 
		  } 
		} catch(Exception e) {
		  //primeFunction = null;
		  //primeP = null;
		}
		
		return primeP;
	}


	/**
	* Creates the initial population. This is an array of individuals.
	* Its size is sizeOfPopulation.
	* The program slot of each individual is initialized
	* to a suitable random program.
	* 
	* This method with a Random parameter is created for running DGPExp to check the number of duplicates 
	* among randomly generated populations.
	*/
        //void createPopulation() {
        /*
        public Vector createPopulation(Random r) {
	  
	        random = r;
	        
		int allowableDepth;
		boolean fullP;
		//Hashtable generation0UniquifierTable = new Hashtable();
		Vector generation0UniquifierVector = new Vector();
		
		population = new Individual[populationSize.intValue()];
		int minimumDepthOfTrees = 1;
		boolean fullCycleP = false;
		int maxDepthForNewIndivs = maxDepthForNewIndividuals.intValue();
		int attemptsAtThisIndividual = 0;
		int individualIndex = 0;
		while (individualIndex < population.length) {
			switch (((MethodOfGeneration)methodOfGeneration.getSelection()).getValue()) {
			case MethodOfGeneration.FULL:
					allowableDepth = maxDepthForNewIndivs;
					fullP = true;
					break;
			case MethodOfGeneration.GROW:
					allowableDepth = maxDepthForNewIndivs;
					fullP = false;
					break;
			case MethodOfGeneration.RAMPED_HALF_AND_HALF:
					//	With each new individual, the allowed depth is ramped up
					//	until it reaches the maximum. Then it is reset to the minimum
					//	depth again.
					//	At this time, we toggle between full trees and grown trees,
					//	so that half of the trees are full and the other half grown.
					//	Note that the switch occurs only at the first attempt to create
					//	a new tree. This is to avoid unecessary toggling when several
					//	attempts are necessary for the same individual. 
					allowableDepth =
							minimumDepthOfTrees +
							(individualIndex %
							(maxDepthForNewIndivs - minimumDepthOfTrees + 1));
					if (attemptsAtThisIndividual == 0 &&
							individualIndex % 
							(maxDepthForNewIndivs - minimumDepthOfTrees + 1) == 0) {
						fullCycleP = !fullCycleP;
					}
					fullP = fullCycleP;
					break;
			default:
					allowableDepth = maxDepthForNewIndivs;
					fullP = false;
					break;
			}

			Program newProgram = createIndividualProgram(
					allowableDepth,  
					true,
					fullP);

			//	Check if we have already created this program.
			//	If not then store it and move on.
			//	If we have then try again.
			//	The easiest way to compare two programs is to
			//	look at their printed representation:
			//
			//System.out.println("??? going to convert program to string");
			//String hashKey = newProgram.toString(0);
			String programKey = newProgram.toString(0);
			//if (!generation0UniquifierTable.containsKey(hashKey)) {
			if (!generation0UniquifierVector.contains(programKey)) {  
				population[individualIndex] = new Individual(newProgram);
				individualIndex++;
				//generation0UniquifierTable.put(hashKey, newProgram);
				generation0UniquifierVector.addElement(programKey);
				attemptsAtThisIndividual = 0;
			} else {
				attemptsAtThisIndividual++;
				if (attemptsAtThisIndividual > 20) {
					//	This depth has probably filled up, so bump the
					//	depth counter and keep the max depth in line with
					//	the new minimum:
					//
					minimumDepthOfTrees++;
					maxDepthForNewIndivs = Math.max(maxDepthForNewIndivs, minimumDepthOfTrees);
					attemptsAtThisIndividual = 0;
				}
			}
		}

		return generation0UniquifierVector;
	}
	*/


	/**
	* Creates the initial population. This is an array of individuals.
	* Its size is sizeOfPopulation.
	* The program slot of each individual is initialized
	* to a suitable random program.
	*/
        //void createPopulation() {
        public Vector createPopulation() {
		int allowableDepth;
		boolean fullP;
		//Hashtable generation0UniquifierTable = new Hashtable();
		Vector generation0UniquifierVector = new Vector();
		
		population = new Individual[populationSize.intValue()];
		int minimumDepthOfTrees = 1;
		boolean fullCycleP = false;
		int maxDepthForNewIndivs = maxDepthForNewIndividuals.intValue();
		int attemptsAtThisIndividual = 0;
		int individualIndex = 0;

		/*Creating prime program for LosAltosHills trail
		nodeCount = 0;
		Program primePrg = createPrimeIndividualProgram();  
		  
		//System.out.println("primeFunc created and assigned to primePrg");

		String primePrgKey = primePrg.toString(0);
		System.out.println("Prime Individual :\n" + primePrgKey);
		population[individualIndex] = new Individual(primePrg);
		individualIndex ++;
		generation0UniquifierVector.addElement(primePrgKey); 
		*/

		//creating prime individual for SantaFe trail
		/*
		try {
		  Class primeCls = ((ProgramChoice) functionSet.getSelectedItem(0)).value();
		  Function primeFunc = (Function)primeCls.newInstance();
		  //Class prime1Arg0Cls = ((ProgramChoice) terminalSet.getSelectedItem(2)).value();
		  primeCls = ((ProgramChoice) terminalSet.getSelectedItem(2)).value();
		  primeFunc.arg[0] = (Program)primeCls.newInstance();
		  primeCls = ((ProgramChoice) functionSet.getSelectedItem(2)).value();
		  primeFunc.arg[1] = (Program)primeCls.newInstance();

		  Function prime1Func = (Function) primeFunc.arg[1];
		  prime1Func.arg[0] = (Program)(((ProgramChoice) terminalSet.getSelectedItem(0)).value()).newInstance();
		  prime1Func.arg[1] = (Program)(((ProgramChoice) functionSet.getSelectedItem(1)).value()).newInstance();

		  Function prime2Func = (Function) prime1Func.arg[1];
		  prime2Func.arg[0] = (Program)(((ProgramChoice) functionSet.getSelectedItem(0)).value()).newInstance();
		  
		  Function prime3Func = (Function) prime2Func.arg[0];
		  prime3Func.arg[0] = (Program)(((ProgramChoice) terminalSet.getSelectedItem(2)).value()).newInstance();
		  prime3Func.arg[1] =  (Program)(((ProgramChoice) terminalSet.getSelectedItem(1)).value()).newInstance();
		
		  //System.out.println("IFA, Move, Pn3, left, pn2, ifa, move, right created. ");
		  prime2Func.arg[0] = prime3Func;
		  
		  //System.out.println("prime3Func allocated to prime2Func.arg[0]");
		  
		  prime2Func.arg[1] = (Program)(((ProgramChoice) functionSet.getSelectedItem(1)).value()).newInstance();

		  //System.out.println("pgn2 created.");

		  Function prime4Func = (Function) prime2Func.arg[1];

		  //System.out.println("prime2Func.arg[1] assigned to prime4Func");

		  prime4Func.arg[0] = (Program)(((ProgramChoice) terminalSet.getSelectedItem(1)).value()).newInstance();
		  prime4Func.arg[1] = (Program)(((ProgramChoice) functionSet.getSelectedItem(1)).value()).newInstance();

		  //System.out.println("prime4Func's right and pn2 created.");

		  Function prime5Func = (Function) prime4Func.arg[1];
		  prime5Func.arg[0] = (Program)(((ProgramChoice) terminalSet.getSelectedItem(0)).value()).newInstance();
		  prime5Func.arg[1] = (Program)(((ProgramChoice) terminalSet.getSelectedItem(1)).value()).newInstance();

		  //System.out.println("prime5Func created. left & move created.");
		  prime4Func.arg[1] = prime5Func;
		  prime2Func.arg[1] = prime4Func;
		  prime1Func.arg[1] = prime2Func;
		  
		  //System.out.println("prime5Func, prime4Func, prime2Func allocated back to their respective args");

		  prime1Func.arg[2] = (Program)(((ProgramChoice) functionSet.getSelectedItem(1)).value()).newInstance();

		  Function prime6Func = (Function) prime1Func.arg[2];
		  prime6Func.arg[0] = (Program)(((ProgramChoice) functionSet.getSelectedItem(0)).value()).newInstance();

		  //System.out.println("prime6Func's arg0 ifa created.");

		  Function prime7Func = (Function) prime6Func.arg[0];
		  prime7Func.arg[0] = (Program)(((ProgramChoice) terminalSet.getSelectedItem(2)).value()).newInstance();
		  prime7Func.arg[1] = (Program)(((ProgramChoice) terminalSet.getSelectedItem(0)).value()).newInstance();
		  prime6Func.arg[0] = prime7Func;
		  
		  //System.out.println("prime7Func created & completed and assigned back to prime6Func.arg[0]");

		  prime6Func.arg[1] = (Program)(((ProgramChoice) terminalSet.getSelectedItem(2)).value()).newInstance();
		  
		  prime1Func.arg[2] = prime6Func;
		  primeFunc.arg[1] = prime1Func;
		  
		  //System.out.println("prime6Func & prime1Func completed & assigned back to their args.");

		  Program primePrg = primeFunc;
		  
		  //System.out.println("primeFunc created and assigned to primePrg");

		  String primePrgKey = primePrg.toString(0);
		  System.out.println("Prime Individual :\n" + primePrgKey);
		  population[individualIndex] = new Individual(primePrg);
		  individualIndex ++;
		  generation0UniquifierVector.addElement(primePrgKey); 

	} catch (Exception e) { 
	  System.out.println("Creation of prime program failed.");
	  Program primePrg = null;
	}
	*/

        while (individualIndex < population.length) {
			switch (((MethodOfGeneration)methodOfGeneration.getSelection()).getValue()) {
			case MethodOfGeneration.FULL:
					allowableDepth = maxDepthForNewIndivs;
					fullP = true;
					break;
			case MethodOfGeneration.GROW:
					allowableDepth = maxDepthForNewIndivs;
					fullP = false;
					break;
			case MethodOfGeneration.RAMPED_HALF_AND_HALF:
					//	With each new individual, the allowed depth is ramped up
					//	until it reaches the maximum. Then it is reset to the minimum
					//	depth again.
					//	At this time, we toggle between full trees and grown trees,
					//	so that half of the trees are full and the other half grown.
					//	Note that the switch occurs only at the first attempt to create
					//	a new tree. This is to avoid unecessary toggling when several
					//	attempts are necessary for the same individual. 
					allowableDepth =
							minimumDepthOfTrees +
							(individualIndex %
							(maxDepthForNewIndivs - minimumDepthOfTrees + 1));
					if (attemptsAtThisIndividual == 0 &&
							individualIndex % 
							(maxDepthForNewIndivs - minimumDepthOfTrees + 1) == 0) {
						fullCycleP = !fullCycleP;
					}
					fullP = fullCycleP;
					break;
			default:
					allowableDepth = maxDepthForNewIndivs;
					fullP = false;
					break;
			}

			Program newProgram = createIndividualProgram(
					allowableDepth,  
					true,
					fullP);

			//	Check if we have already created this program.
			//	If not then store it and move on.
			//	If we have then try again.
			//	The easiest way to compare two programs is to
			//	look at their printed representation:
			//
			//System.out.println("??? going to convert program to string");
			//String hashKey = newProgram.toString(0);
			String programKey = newProgram.toString(0);
			//if (!generation0UniquifierTable.containsKey(hashKey)) {
			if (!generation0UniquifierVector.contains(programKey)) {  
				population[individualIndex] = new Individual(newProgram);
				individualIndex++;
				//generation0UniquifierTable.put(hashKey, newProgram);
				generation0UniquifierVector.addElement(programKey);
				attemptsAtThisIndividual = 0;
			} else {
				attemptsAtThisIndividual++;
				if (attemptsAtThisIndividual > 20) {
					//	This depth has probably filled up, so bump the
					//	depth counter and keep the max depth in line with
					//	the new minimum:
					//
					minimumDepthOfTrees++;
					maxDepthForNewIndivs = Math.max(maxDepthForNewIndivs, minimumDepthOfTrees);
					attemptsAtThisIndividual = 0;
				}
			}
		}

		return generation0UniquifierVector;
	}

	static final Condition isProgram = new IsProgram();
	static final Condition isFunction = new IsFunction();

	/**
	* Produces two new individuals by crossing two parents.
	* First, a crossover point is selected randomly in each parent.
	* Experience shows that it is more useful to select inner nodes
	* (functions, not terminals) as crossover points.
	* The ratio of cuts at inner points vs. cuts at terminals is fixed
	* in this function.
	* Then the two parents are cut at their crossover points, giving
	* two tree fragments and two cut-off subtrees. The two subtrees
	* are swapped and spliced with the fragments again.
	* We have to make sure that the newly formed trees do not exceed a
	* certain depth. If they do, one of the parents is used as offspring.
	*/
	Program[] crossover(Program male, Program female) {
		double CrossoverAtFunctionFraction = 0.9;
		boolean crossoverAtFunction;
		//	Make copies of the parents first, because they will be destructively modified:
		Program[] offspring = new Program[2];
		offspring[0] = (Program)male.clone();
		offspring[1] = (Program)female.clone();

		int malePoint, femalePoint;
		TreeHook maleHook, femaleHook;
		crossoverAtFunction = (random.nextDouble() < CrossoverAtFunctionFraction);
		if (crossoverAtFunction) {
			malePoint = random.nextInt(male.countNodes(isFunction));
			maleHook = getSubtree(offspring[0], malePoint, isFunction);
		} else {
			malePoint = random.nextInt(male.countNodes());
			maleHook = getSubtree(offspring[0], malePoint, isProgram);
		}
		crossoverAtFunction = (random.nextDouble() < CrossoverAtFunctionFraction);
		if (crossoverAtFunction) {
			femalePoint = random.nextInt(female.countNodes(isFunction));
			femaleHook = getSubtree(offspring[1], femalePoint, isFunction);
		} else {
			femalePoint = random.nextInt(female.countNodes());
			femaleHook = getSubtree(offspring[1], femalePoint, isProgram);
		}

		//	Modify the new individuals by smashing in the (copied) subtree from the old individual.
		if (maleHook.parent == null) {
			offspring[0] = femaleHook.subtree;
		} else {
			maleHook.parent.arg[maleHook.childIndex] = femaleHook.subtree;
		}
		if (femaleHook.parent == null) {
			offspring[1] = maleHook.subtree;
		} else {
			femaleHook.parent.arg[femaleHook.childIndex] = maleHook.subtree;
		}

		//	Make sure that the new individuals aren't too big.
		validateCrossover(male, female, offspring);
		return offspring;
	}

	/**
	* @return the depth of the deepest branch of the tree
	*/
	int maxDepthOfTree(Program tree) {
		int maxDepth = 0;
		if (tree instanceof Function) {
			for (int a = 0; a < ((Function)tree).arg.length; a++) {
				Program s = ((Function)tree).arg[a];
				int depth = maxDepthOfTree(s);
				maxDepth = Math.max(maxDepth, depth);
			}
			return (1 + maxDepth);
		} else {
			return 0;
		}
	}

	/**
	* Given the parents and two offsprings from a crossover operation
	* check to see whether we have exceeded the maximum
	* allowed depth. If a new individual has exceeded the maximum depth
	* then one of the parents is used.
	*/
	void validateCrossover(Program male, Program female,
		Program[] offspring) {
		int depth;
		for (int i = 0; i < offspring.length; i++) {
			if (offspring[i] == null) {
				depth = 0;
			} else {
				depth = maxDepthOfTree(offspring[i]);
			}
			if (depth < 1 || depth > maxDepthForIndividualsAfterCrossover.intValue()) {
				int whichParent = random.nextInt(2);
				if (whichParent == 0) {
					offspring[i] = (Program)male.clone();
				} else {
					offspring[i] = (Program)female.clone();
				}
			}
		}
	}

	/**
	* Mutates the argument program by picking a random point in
	* the tree and substituting in a brand new subtree created in
	* the same way that we create the initial random population.
	* @return a mutated copy of the original program
	*/
	Program mutate(Program program) {
		//	Pick the mutation point.
		int mutationPoint = random.nextInt(program.countNodes());
		//	Create a brand new subtree.
		Program newSubtree = createIndividualProgram(
				maxDepthForNewSubtreesInMutants.intValue(),
				true,
				false);
		Program newProgram = (Program)program.clone();
		//	Smash in the new subtree.
		TreeHook hook = getSubtree(program, mutationPoint, isProgram);
		if (hook.parent == null) {
			newProgram = hook.subtree;
		} else {
			hook.parent.arg[hook.childIndex] = newSubtree;
		}
		return newProgram;
	}

	/**
	* Controls the actual breeding of the new population.  
	* Loops through the population executing each operation
	* (e.g., crossover, fitness-proportionate reproduction,
	* mutation) until it has reached the specified fraction.  
	* The new programs that are created are stashed in newPrograms
	* until we have exhausted the population, then we copy the new
	* individuals into the old ones.
	*/
	void breedNewPopulation() {
		Program[] newPrograms;
		double fraction;
		int index;
		Individual individual1, individual2;
		int i;
		double sumOfFractions = this.crossoverFraction.doubleValue() +
				this.fitnessProportionateReproFraction.doubleValue() +
				this.mutationFraction.doubleValue();
		double crossoverFraction = this.crossoverFraction.doubleValue() / sumOfFractions;
		double reproductionFraction = this.fitnessProportionateReproFraction.doubleValue() / sumOfFractions;
		// double mutationFraction = this.mutationFraction.get() / sumOfFractions;

		newPrograms = new Program[population.length];
		fraction = 0.0;
		index = 0;

		//	A hint from Jrg Frhlich: keep the best individual all the time:
		// Elitism is not used here according to Koza's GP book.
		
		if (((ElitismSelection)elitismChosen.getSelection()).getValue() == ElitismSelection.ELITISM_USED) {
		  
		  newPrograms[index] = (Program)bestOfRunIndividual.program.clone();
		  System.out.println("Population[" + index + "] --- the best individual is cloned.");
		  index++;
		}
		
		while (index < population.length) {
			fraction = (double)index / (double)population.length;
			individual1 = findIndividual();
			if (fraction < crossoverFraction) {
				individual2 = findIndividual();
				Program[]	offspring = crossover(individual1.program, individual2.program);
    		newPrograms[index] = offspring[0];
		        //System.out.println("Population[" + index +"] produced by Crossover operation.");

				index++;
				if (index < population.length) {
    			newPrograms[index] = offspring[1];
			//System.out.println("Population[" + index +"] produced by Crossover operation.");

					index = index++;
				}
			} else {
				if (fraction < 	reproductionFraction + crossoverFraction) {
					newPrograms[index] = (Program)individual1.program.clone();
					//System.out.println("Population[" + index + "] produced by Cloning operation.");

					index ++;
				} else {
					newPrograms[index] = mutate(individual1.program);

					//System.out.println("Population[" + index + "] produced by Mutation operation.");

					index++;
				}
			}
		}
		for (index = 0; index < population.length; index++) {
			population[index].program = newPrograms[index];
		}
	}


  /**
  * Save current population into file for further use.
  */    
  void savePopulation() {

    /*
    System.out.println("Starting saveFileThread");
    Thread sthread = new saveFileThread(applet_DGP.frame,this);
    sthread.start();
    */

    //Dialog d = new SaveFileDialog(applet_DGP.frame,this);
    Dialog d = new SaveFileDialog(Dialog.findFirstFrame(applet_DGP),this);
    d.show();
    d.pack();

    /*
    SaveFileFrame sframe = new SaveFileFrame(this);
    sframe.show();
    sframe.hide();

    sframe.show();
    ////frame.resize(frame.insets().left + frame.insets().right  + 500,
    ////			 frame.insets().top  + frame.insets().bottom + 400);
    sframe.resize(sframe.insets().left + sframe.insets().right  + 530,
		  sframe.insets().top  + sframe.insets().bottom + 300);
    sframe.pack();
    sframe.setLocation(30,30);
    */

    //m = new ThreadMonitor();
    //System.out.println("Thread Monitor created and started");
    // m.waitUntilFinishes();

     System.out.println("fileName specified by user is " + fileName);

    /*
    //create a file dialog to query the user for a filename.
    FileDialog f = new FileDialog(applet_DGP.frame,"Save Current Population to File", FileDialog.SAVE);
    f.show();
    fileName = f.getFile();
    //popFile = new File(fileName);
    */

    Hashtable saveObj = new Hashtable();
    saveObj.put("Population", population);
    saveObj.put("Seed", new Integer(SEED));
    saveObj.put("ClientID", clientID);
    saveSettingsObj(saveObj); 

    if (fileName != null) {
      try {
	notifyObservers(new GPMessageStatus("Saving to file\n" + fileName + " ..."));
	//Create the necessary output streams to save the population
	FileOutputStream fos = new FileOutputStream(fileName); //Save to file
	GZIPOutputStream gzos = new GZIPOutputStream(fos); //Compress
	ObjectOutputStream out = new ObjectOutputStream(gzos); //Save objects
	//*out.writeObject(population);
	out.writeObject(saveObj);
	out.flush();
	out.close();
      }
      // Print out exceptions. We should really display them in a dialog ...
      catch (IOException e) { System.out.println(e); }
    } else {
      notifyObservers(new GPMessageStatus("Population and seed not saved.\nThis run cannot be continued."));
    }
  }


  /**
  * Read current population from file to continue from where it stops.
  */ 
  void loadPopulation(String fname) {
    /*
    //Create a file dialog to query the user for a filename.
    FileDialog f = new FileDialog(applet_DGP.frame,"Load population from file", FileDialog.LOAD);
    f.show();
    String filename = f.getFile();
    */
    //&if (fname != null) {
    //&try {
	//Create the necessary input streams to load the population
    //&FileInputStream fis = new FileInputStream(fname); //Read from file
    //&GZIPInputStream gzis = new GZIPInputStream(fis); //Uncompress
    //&ObjectInputStream in = new ObjectInputStream(gzis); //Read objects
	//Read in an object. It should be an array of Individuals
	//*Individual[] pop = (Individual[])in.readObject();
    //&Hashtable runStateTable = (Hashtable)in.readObject();
    //&in.close();
	population = (Individual[])runStateTable.get("Population");
	//&SEED = ((Integer)runStateTable.get("Seed")).intValue();
	//&clientID = (String)runStateTable.get("ClientID");
	//&restoreSettingsObj(runStateTable);

	//population = pop;
	//System.out.println("population size : " + population.length);
	//&}	
      // Print out exceptions. We should really display them in a dialog ...
	//&catch (IOException e) { System.out.println(e); }
	//&catch (ClassNotFoundException c) {System.out.println(c); }
	//&}
  }


  void loadGPSettings(String fname) {

      if (fname != null) {
      try {
	//Create the necessary input streams to load the population
	FileInputStream fis = new FileInputStream(fname); //Read from file
	GZIPInputStream gzis = new GZIPInputStream(fis); //Uncompress
	ObjectInputStream in = new ObjectInputStream(gzis); //Read objects
	//Read in an object. It should be an array of Individuals
	//*Individual[] pop = (Individual[])in.readObject();
	runStateTable = (Hashtable)in.readObject();
	in.close();

	SEED = ((Integer)runStateTable.get("Seed")).intValue();
	clientID = (String)runStateTable.get("ClientID");
	restoreSettingsObj(runStateTable);

	//population = pop;
	//System.out.println("population size : " + population.length);
      }	
      // Print out exceptions. We should really display them in a dialog ...
      catch (IOException e) { System.out.println(e); }
      catch (ClassNotFoundException c) {System.out.println(c); }
    }
  }

	/**
	* Clean out the statistics in each individual in the
	* population. 
	*/
	void zeroizeFitnessMeasuresOfPopulation() {
		for (int i = 0; i < population.length; i++) { 
			population[i].standardizedFitness = 0.0;
			population[i].adjustedFitness = 0.0;
			population[i].normalizedFitness = 0.0;
			population[i].hits = 0;
			//    	population[i].sent = false;
		}
	}

	/**
	* Loops over the individuals in the population evaluating and
	* recording the fitness and hits.
	*/
	void evaluateFitnessOfPopulation() {
	        solutionFound = false;
		//System.out.println("population length is " + population.length);
		for (int i = 0; i < population.length; i++) {
		  //System.out.println("Evaluating program : " + population[i].program.toString(0));
		  //fitnessFunction(population[i], i);
		  //RealPoint[] evalResults = new RealPoint[fitnessCases.get().length];
		  //evalResults = problem.fitnessFunction(population[i],this);
		  //*System.out.println("\n??? program is : \n" + population[i].program.toString(0) + "\n");

		  /* If there are more than 1 solution in a population, the last found will be used. It does not
		     matter which one we use because both of them are solutions. 
		     If solution is not found (maxHits not reached), solution will be set to null. */ 
		  
		  boolean solFound = problem.fitnessFunction(population[i],this,i);
		  
		  if (solFound) {
		    solutionFound = true;
		    individualNrOfSolution = i;
		  }
		  
		  //notifyObservers(new GPMessageEvaluatingIndividual(currentGeneration, i, evalResults));
		}
	}

	/**
	* Computes the normalized and adjusted fitness of each
	* individual in the population.
	*/
         /*
	void normalizeFitnessOfPopulation() {

		sumOfAdjustedFitnesses = 0.0;
		for (int i = 0; i < population.length; i++) {
			//	Set the adjusted fitness.
			population[i].adjustedFitness = 
    			1.0 / (population[i].standardizedFitness + 1.0);
			//	Add up the adjusted fitnesses so that we can normalize them.
			sumOfAdjustedFitnesses =
					sumOfAdjustedFitnesses + population[i].adjustedFitness;
		}
		//	Loop through population normalizing the adjusted fitness.
		for (int i = 0; i < population.length; i++) {
			population[i].normalizedFitness =
					population[i].adjustedFitness / sumOfAdjustedFitnesses;
		}
		
	}
	*/

  void createAndExportIndivArray(int exportQuantity) {

    Individual[] IndArray = new Individual[exportQuantity];
    //Hashtable IndTable = new Hashtable();
    
    //IndArray = new Individual[exportQuantity];
    Individual[] iArray;
		  counter = 0;
		  int index = 0;
		  boolean allSent = false;
		  //int selCount = 0;
		  selCount = 0;

		  for (int indCount = 0 ; indCount < exportQuantity; indCount++) {

		    if (! allSent) {
		      System.out.println("Looking for the best individual(s) that has not been sent");
		      //System.out.println("Sent flag of population[" + index + "] : " + population[index].sent);
		    
		      //while (index < population.length) {
		      while (population[index].sent) {
			//the sent variable for the solution individual in population has been set to true in the 
			//problem specific method 'fitnessFunction'. Thus, the solution will not be selected here.
			
			//System.out.println("population[" + index + "].sent is true. Inside loop to search for next unsend individual.");
			index ++;
			
			if (index == population.length) {
			  break;
			}
			  
		      }
		      //   }
		  
		  // set sent to true before exporting so that individual cannot be resent by this W.S as well as other W.S
		      if (index < population.length) {
			selCount ++;
			population[index].sent = true;
			
			System.out.println("Population[" + index + "] was selected for export and its 'send' flag is set to true.");
			
			//System.out.println("Sent flag of population[" + index + "] : " + population[index].sent);
			
			IndArray[indCount] = population[index];
			//IndTable.put(population[index].toString(), population[index]);	    
			System.out.println("Population[" + index + "] was inserted into individual array.");
			//System.out.println("Population[" + index + "] was inserted into individual table.");
		      }
		      else {
			System.out.println("All individuals sent !");
			allSent = true;
		      }
		    }
		  }
		  
		 
		  //if (index < population.length) {
		  //*if (IndArray.length == exportQuantity) {
		  if (selCount == exportQuantity) {
		    System.out.println("Creation of Individual Array completed.");
		    exportIndividual(IndArray);
		    /*
		    if (! IndTable.isEmpty()) {
		      exportIndividual(IndTable);
		    }
		    */
		  }
		  else 
		    if (selCount != 0) {
		      iArray = new Individual[selCount];
		      for (int i=0; i < selCount; i++) {
			iArray[i] = IndArray[i];
		      }
		      IndArray = new Individual[selCount];
		      for (int k=0; k < selCount; k++) {
			IndArray[k] = iArray[k];
		      }

		      System.out.println("Creation of partial individual Array completed.");
		      //exportIndividual(iArray);
		      exportIndividual(IndArray);
		    }
		    
  }

  //void exportIndividual(Individual goodIndividual) {
  void exportIndividual(Individual[] individualArray) {
  //void exportIndividual(Hashtable individualTable) {

    String status;
    String setStatus;

    /*
    System.out.println("Exporting individualArray containing " + individualArray.length + " individuals to recipients defined in ClientList file.");

    //  System.setSecurityManager(new RMISecurityManager());

    BufferedReader d = null ;	
    String line = "" ;	    
    // String thisHost = "";
    String subLine = "";
    
    clientFile = new String("/home/students/acs97/fsc/fs-project2/GPsrc/SymRegrs/" + fileName);
    //clientFile = new String("/tmp/fsc/" + fileName);

    System.out.println("Reading ClientList file for host names of recipients ...");
    System.out.println("ClientList file is : " + clientFile);

    try {
	d = new BufferedReader(new FileReader(clientFile));
	} catch(FileNotFoundException e) {
	 	System.out.println("file not found...");
		//log.println("file not found...");
		}

	boolean cont = true ;

	while (cont == true) {
	  try {
	    line = (d.readLine()) ;
	  } catch(IOException e) {
	        System.out.println("readLine for ClientList file error: " + e);
		//log.println("readLine for ClientList file error: " + e);
		cont = false ;
	  }
	  */
	/*
	try {
	  thisHost = new String(InetAddress.getLocalHost().getHostName()) ;
	} 
	catch(java.net.UnknownHostException e) {
	  	  System.out.println("Unknown host exception in 'exportIndividual()' ! ");
		  log.println("Unknown host exception in 'exportIndividual()' ! ");
	}
	*/
	/*
	System.out.println("thisHost : " + thisHost);
	log.println("thisHost : " + thisHost);
	System.out.println("line : " + line);
	log.println("line : " + line);
	*/
    /*
	  if (line !=null) {
	    subLine = line.substring(0,line.indexOf("."));
	  }
	  */
	/*
	System.out.println("subLine : " + subLine);
	log.println("subLine : " + subLine);
	*/

    // if ((line != null) && (!subLine.equalsIgnoreCase(thisHost))) {
    //    System.out.println("Sending individual to " + line);

    if (individualArray != null) {
      System.out.println("individualArray is not null. No. of elements is " + individualArray.length);
    }
    else {
      System.out.println("individualArray is null !");
    }


    Hashtable sendObj = new Hashtable();
    sendObj.put("ClientID", clientID);

    if ((! runComplete) && (! suddenEnd)) {
      sendObj.put("RecvIndNum",new Integer(imigrantQueueSize.intValue()));
    }
    else {
      sendObj.put("RecvIndNum",new Integer(0));
    }

    sendObj.put("IndArray", individualArray);


    if (sendObj != null) {
      System.out.println("sendObj is not null. No. of elements is " + sendObj.size());
    }
    else {
      System.out.println("sendObj is null !");
    }


    System.out.println("Sending individual to server");

    if (DGPClient.experiment) {
	servletURL = servletHost + "PopExpServlet?ProbName=" + applet_DGP.probName ;
    }
    else {
	servletURL = servletHost + "Pop25Servlet?ProbName=" + applet_DGP.probName ;
    }
	
	  System.out.println("servletURL : " + servletURL);
    
	  replyObj = null;

	  //if (SEED != 12345) {
	  //if (SEED != applet_DGP.defaultSEED) {
	  lastSendSucceed = false;
	  NoConnection = true;
	  
	  sb = new StringBuffer();
	  if (individualArray != null) {       
	    System.out.println("Individual(s) sent have hit counts of\n"); 
	    sb.append("Individual(s) sent have hit counts of\n");
	    for (int e=0; e < individualArray.length; e++) {
	      sb.append(individualArray[e].hits + " ");
	      System.out.println(individualArray[e].hits + " ");
	    }
	  }
	  else {
	    sb.append("");
	  }
	 
	    	    
	  while (NoConnection) {
	      notifyObservers(new GPMessageStatus("Sending intermediate results to server.\n" + sb.toString()));
	      System.out.println("Sending intermediate results to server.\n" + sb.toString());
	      new ThreadMonitor().waitUntilTimeOut(1200);
	      /*
	      try {
		System.out.println("Waiting for 5secs");
		m.wait(500);
	      }
	      catch(InterruptedException q) {
		System.out.println("InterruptedException at wait(500) : " + q);
	      }
	      */
	      System.out.println("Sending results to server. " + sb.toString());
	      try {
		System.out.println("going to run putObject for individual");
		replyObj = (Individual[])putObject(servletURL, sendObj);

		if (replyObj != null) {
		  sb = new StringBuffer();
		  sb.append("Individual(s) received have hit counts of\n");
		  for (int f=0; f < replyObj.length; f++) {
		    sb.append(replyObj[f].hits + " ");
		  }
		}
		notifyObservers(new GPMessageStatus("Receiving individuals from server.\n" + sb.toString()));
		new ThreadMonitor().waitUntilTimeOut(2000);
		//m.wait(500);
		System.out.println("*** Results successfully sent to server ! ***\n" + sb.toString());		
		NoConnection = false;
		lastSendSucceed = true;
	      }
	      catch(MalformedURLException m) {
		System.out.println("MalformedURLException at putObject() !");
		NoConnection = false;
	      } 
	      catch (OptionalDataException o) {
		System.out.println("OptionalDataException at getObject() !");
		NoConnection = false;
	      } 
	      catch (ClassNotFoundException c) {
		System.out.println("ClassNotFoundException at getObject() !");
		NoConnection = false;
	      }
	      /*
	      catch (InterruptedException u) {
		System.out.println("InterruptedException at wait(500) : " + u);
	      }
	      */
	      catch(IOException e) {
		//System.out.println("Run not completed;Exception in exportIndividual: " + e);
		if ((! runComplete) && (! suddenEnd)) {
		//new ThreadMonitor().waitUntilTimeOut(1200);
		notifyObservers(new GPMessageStatus("Cannot export individual.\nModem may NOT be ON.\nExport abandoned."));
		NoConnection = false;
		}
		else if ((! runComplete) && (suddenEnd)) {
		  System.out.println("Run terminated;Exception in exportIndividual: " + e);
		  notifyObservers(new GPMessageStatus("RUN TERMINATED !\nCannot export individual.\nModem may NOT be ON."));
		  ModemDialog n = new ModemDialog(Dialog.findFirstFrame(applet_DGP),"*** RUN TERMINATED ! ***\n\nCould not connect to Server to send run results !\n\nPlease check that your modem is powered on and hit the <OK> button.\n\nIf, however, you do not wish to send the results back, hit the <Cancel> button.", "Network Problem");
		  n.show();
		  n.pack();
		  if (! n.getDialogAns()) {
		    NoConnection = false;
		    notifyObservers(new GPMessageStatus("Cannot export individual.\nModem may NOT be ON.\nExport abandoned."));
		  }
		}
		else {
		  System.out.println("Run completed;Exception in exportIndividual: " + e);
		  notifyObservers(new GPMessageStatus("RUN COMPLETED !\nCannot export individual.\nModem may NOT be ON."));
		  ModemDialog n = new ModemDialog(Dialog.findFirstFrame(applet_DGP),"*** RUN COMPLETED ! ***\n\nCould not connect to Server to send run results !\n\nPlease check that your modem is powered on and hit the <OK> button.\n\nIf, however, you do not wish to send the results back, hit the <Cancel> button.", "Network Problem");
		  n.show();
		  n.pack();
		  if (! n.getDialogAns()) {
		    NoConnection = false;
		    notifyObservers(new GPMessageStatus("Cannot export individual.\nModem may NOT be ON.\nExport abandoned."));
		  }
		}
	      } //catch	      
	  } //while loop


	    /*
	    while (NoConnection) {
	      notifyObservers(new GPMessageStatus("Sending results back to server ..."));
	      System.out.println("noConnection : " + NoConnection);
	      
	      try {
		
		//if ((applet_DGP.i == null) && (SEED != 12345)) {
		if (applet_DGP.i == null) {
		  System.out.println("remote object is null. Naming.lookup for remote object");
		  if (! applet_DGP.Application) {
		    String appletURL = "//" + applet_DGP.getCodeBase().getHost() + "/importImigrant";
		    System.out.println("appletURL : " + appletURL); 
		    //i = (imigrant)Naming.lookup("//" + applet_DGP.getCodeBase().getHost() + "/importImigrant");
		    applet_DGP.i = (imigrant)Naming.lookup("//" + "hamm.cs.bham.ac.uk" + "/importImigrant");
		  }
		  else { 
		    applet_DGP.i = (imigrant)Naming.lookup("rmi://" + "hamm.cs.bham.ac.uk" + "/importImigrant");
		  }
		}
		*/

	      /*
	      //imigrant i;
	      System.out.println("application ? " + applet_DGP.Application);
	      //imigrant i = (imigrant)Naming.lookup("rmi://" + "studentweb.cs.bham.ac.uk" + "/importImigrant");
	      if (! applet_DGP.Application) {
		String appletURL = "//" + applet_DGP.getCodeBase().getHost() + "/importImigrant";
		System.out.println("appletURL : " + appletURL); 
		//i = (imigrant)Naming.lookup("//" + applet_DGP.getCodeBase().getHost() + "/importImigrant");
		i = (imigrant)Naming.lookup("//" + "rex.cs.bham.ac.uk" + "/importImigrant");
	      }
	      else { 
		i = (imigrant)Naming.lookup("rmi://" + "rex.cs.bham.ac.uk" + "/importImigrant");
	      }
	      //imigrant i = (imigrant)Naming.lookup("rmi://" + line + "/importImigrant");
	      //String status = i.receiveImigrant(goodIndividual,thisHost);
	      */
	      //status = i.receiveImigrant(individualArray,thisHost);
	    /*
	      System.out.println("trying to invoke remote method receiveImigrant");

		status = applet_DGP.i.receiveImigrant(individualArray,thisHost);
		System.out.println(status);
		
		if (toSetGenOfSolution) {
		  //setStatus = i.setGenerationOfSolution(solutionFirstFoundGenNr,thisHost);
		  //setStatus = i.setGenerationOfSolution(genNrOfSolution,thisHost);
		  setStatus = applet_DGP.i.setGenerationOfSolution(genNrOfSolution,thisHost);
		  System.out.println(setStatus);
		}
		
		NoConnection = false;
		notifyObservers(new GPMessageStatus("Results successfully sent to server."));	
		System.out.println("Results successfully sent to server.");
		lastSendSucceed = true;
	      }
	      catch (Exception e) {
	    //catch (RemoteException e) {
	      System.out.println("Run not completed;Exception in exportIndividual: " + e);
	      if ((! runComplete) && (! suddenEnd)) {
		//new ThreadMonitor().waitUntilTimeOut(1200);
		notifyObservers(new GPMessageStatus("Cannot export individual.\nModem may NOT be ON.\nExport abandoned."));
		NoConnection = false;
	      }
	      else if ((! runComplete) && (suddenEnd)) {
		System.out.println("Run terminated;Exception in exportIndividual: " + e);
		notifyObservers(new GPMessageStatus("RUN TERMINATED !\nCannot export individual.\nModem may NOT be ON."));
		ModemDialog n = new ModemDialog(Dialog.findFirstFrame(applet_DGP),"*** RUN TERMINATED ! ***\n\nCould not connect to Server to send run results !\n\nPlease check that your modem is powered on and hit the <OK> button.\n\nIf, however, you do not wish to send the results back, hit the <Cancel> button.", "Network Problem");
		n.show();
		n.pack();
		if (! n.getDialogAns()) {
		  NoConnection = false;
		  notifyObservers(new GPMessageStatus("Cannot export individual.\nModem may NOT be ON.\nExport abandoned."));
		}
	      }
	      else {
		System.out.println("Run completed;Exception in exportIndividual: " + e);
		notifyObservers(new GPMessageStatus("RUN COMPLETED !\nCannot export individual.\nModem may NOT be ON."));
		ModemDialog n = new ModemDialog(Dialog.findFirstFrame(applet_DGP),"*** RUN COMPLETED ! ***\n\nCould not connect to Server to send run results !\n\nPlease check that your modem is powered on and hit the <OK> button.\n\nIf, however, you do not wish to send the results back, hit the <Cancel> button.", "Network Problem");
		n.show();
		n.pack();
		if (! n.getDialogAns()) {
		  NoConnection = false;
		  notifyObservers(new GPMessageStatus("Cannot export individual.\nModem may NOT be ON.\nExport abandoned."));
		}
	      }
	      } //end of catch
	    } //end of while loop

	    //} //end of if (SEED !=12345)
	    */
	  /*
          URL urldest = null ;
//    	  try { urldest = new URL(linetrim); }
	  try { urldest = new URL(line); }
          catch (MalformedURLException e) {
//		((Coordinator)_aglet).setTheMessage("Malformed URL !" + urldest.toString());
		System.out.println("Malformed URL " + urldest.toString());
		}
		
	itineraryList.addItem(line);
	*/

	    /*} //for if line !=null
	  else 
	    if (line == null) {
	        cont = false;
		System.out.println("eof reached for ClientList file.");
		//log.println("eof reached for ClientList file.");
	     }		

	} //for 'while cont==true loop'
	System.out.println("end of exportIndividual");
	//log.println("end of exportIndividual");
	*/
  }

  synchronized void acceptImigrant() {
    int index = 1 ;
    int popIndex ;
    Individual[] recvIndArray;
    
    /*
    System.out.println("inside acceptImigrant");
    log.println("inside acceptImigrant");
    */

    if (replyObj == null) {
      System.out.println("IndArray received from servlet is empty ! Nothing to accept into population. ");
    }
    else {
      System.out.println("Processing individuals in IndArray received from servlet ...");
      recvIndArray = (Individual[])replyObj;
      for (int i = 0; i < recvIndArray.length; i++) {
	popIndex = population.length - index ;
	population[popIndex] = recvIndArray[i];
	System.out.println("population[" + popIndex + "] replaced by recvIndArray[" + i + "]");
	index ++;
      }
      replyObj = null;
      System.out.println(recvIndArray.length + " individuals were received from server and replaced into population."); 
    }
  }
		
  /*
    if (imigrantTable.isEmpty()) {
      System.out.println("imigrantTable empty ! Nothing to accept into population. ");
      //log.println("imigrantTable empty ! Nothing to accept into population. ");
    }
    else {
      System.out.println("Processing client's queues (Vector entries) in imigrantTable ...");

      Enumeration e = imigrantTable.keys();
      while (e.hasMoreElements()) {
	String client = (String)e.nextElement();
	System.out.println("@ Client : " + client);
	//log.println("@ Client : " + client);
	Vector vec = (Vector)imigrantTable.get(client);
	
	if (vec.isEmpty()) {
	  System.out.println("Vector is empty !");
	}
	else {
	  popIndex = population.length - index ;
	  System.out.println("Initial no. of elements in queue : " + vec.size());
	  //log.println("Initial no. of elements in queue : " + vec.size());
	  System.out.println("Replacing population[" + popIndex + "] with most recently received individual (LIFO) in queue.");

	  population[popIndex] = (Individual)vec.lastElement();
	  vec.removeElement(vec.lastElement());
	  System.out.println("Worst individual in population replaced by most recently received individual in queue. Most recently received individual in queue removed ! ");
	  System.out.println("No. of elements left in imigrantVector: " + vec.size());

	  if (vec.size() == 0) {
	    imigrantTable.remove(client);
	    System.out.println("Queue is empty. Its entry in imigrantTable is removed ! ");
	  }

	  System.out.println("Processing completed for client : " + client);
	
	index ++;
	}
      }
      System.out.println("Finished processing imigrant queues for all clients.");
    }
    sortPopulationByFitness();

  }
  */


  void sendEndResults() {
    
    System.out.println("inside sendEndResults()");

    if (runComplete) {
      System.out.println("Run completed");
      if (! ((counter == 1) && (lastSendSucceed))) { 
	Individual[] solutionArray = {population[0]}; 
	exportIndividual(solutionArray);
      }
    }
    else {
      System.out.println("run not Completed ");
      suddenEnd = true;
      System.out.println("suddenEnd set to true");
      
      System.out.println("Sorting population by fitness in descending order.");
      sortPopulationByFitness();
      
      //if ((canSendResult) && (counter != 1)) { --- ignore checking of counters: difficult to cater for.
      if (canSendResult) {
	System.out.println("can send result ... looking for 1 indv and create array to send");
	//createAndExportIndivArray(numToExport.intValue());
	createAndExportIndivArray(1);
      }
      else 
	if ((IndArray != null) && (selCount > 0) && (! lastSendSucceed)) {
	  System.out.println("IndArray not null and last send not successful. Resending IndArray");
	  exportIndividual(IndArray);
	}
    }
    
    System.out.println("Creating endTable containing end results to be sent back to server.");
    Hashtable endTable = new Hashtable();
    endTable.put("ClientID", clientID);
    System.out.println("endTable created and clientID written.");
    
    //if ((! runComplete) && (! suddenEnd)) {
    if (solutionFound) {
      System.out.println("Putting other info based on solution found");
      endTable.put("SolFound", new Boolean(true));
      //endTable.put("Solution", population[0]);

      endTable.put("Solution", population[0].program.toString(0));

      endTable.put("UserName", solName);
      endTable.put("UserOrg", solOrg);
      endTable.put("UserDesignation", solDesignation);
      endTable.put("Pop_Size", new Integer(populationSize.intValue()));
      endTable.put("Max_Gen", new Integer(maxGenNum.intValue()));
      endTable.put("Export_Interval", new Integer(exportGenNum.intValue()));
      endTable.put("Export_Num", new Integer(numToExport.intValue()));
      endTable.put("Import_Num", new Integer(imigrantQueueSize.intValue()));
      endTable.put("Max_Depth_For_New_Indv", new Integer(maxDepthForNewIndividuals.intValue()));
      endTable.put("Max_Depth_For_Indv_After_Crossover", new Integer(maxDepthForIndividualsAfterCrossover.intValue()));
      endTable.put("Max_Depth_For_New_Subtrees_In_Mutants", new Integer(maxDepthForNewSubtreesInMutants.intValue()));
      endTable.put("Crossover_Prob", new Double(crossoverFraction.doubleValue()));
      endTable.put("Reproduction_Prob", new Double(fitnessProportionateReproFraction.doubleValue()));
      endTable.put("Mutation_Prob", new Double(mutationFraction.doubleValue()));
      endTable.put("Method_Of_Generation", ((MethodOfGeneration)methodOfGeneration.choices[methodOfGeneration.selection]).toString());
      endTable.put("Method_Of_Selection", ((MethodOfSelection)methodOfSelection.choices[methodOfSelection.selection]).toString());
      endTable.put("Elitism", ((ElitismSelection)elitismChosen.choices[elitismChosen.selection]).toString());
      endTable.put("Tournament_selection_group_size", new Integer(tournamentSize.intValue()));		   
      System.out.println("Completed putting info based on solution found");
    }
    else {
      System.out.println("Putting other info based on no solution found");
      endTable.put("SolFound", new Boolean(false));
      endTable.put("Sudden_End", new Boolean(suddenEnd));
      //endTable.put("MaxGen", new Integer(maxGenNum.intValue()));
      endTable.put("FoodEaten", new Integer(population[0].hits));
      System.out.println("Completed putting info based on no solution found");
    }

    if (suddenEnd) {
      endTable.put("EndGen", new Integer(currentGeneration));
    }
    else {
      endTable.put("EndGen", new Integer(currentGeneration -1));
    }
    System.out.println("Completed putting info on endGen");
    
    if (endTable != null) {
      System.out.println("endTable is not null. No. of elements is " + endTable.size());
    }
    else {
      System.out.println("endTable is null !");
    }

    System.out.println("Sending end results to server");

    if (DGPClient.experiment) {
	servletURL = servletHost + "StatExpServlet?ProbName=" + applet_DGP.probName ;	
    }
    else {
	servletURL = servletHost + "Stat10Servlet?ProbName=" + applet_DGP.probName ;
    }

    System.out.println("servletURL : " + servletURL);
    
    //replyObj = null;

    lastSendSucceed = false;
    NoConnection = true;
    String sendStatus = "";
    
    while (NoConnection) {
      notifyObservers(new GPMessageStatus("Sending end results back to server ..."));
      System.out.println("Sending end results back to server ... ");
      try {
	System.out.println("going to run putObject for endResults");
	sendStatus = (String)putObject(servletURL, endTable);
	notifyObservers(new GPMessageStatus("End results successfully sent to server."));
	System.out.println("*** End results successfully sent to server ! ***");
	NoConnection = false;
	lastSendSucceed = true;
	
	String thankMsg; 
	if (! (sendStatus.equals(""))) {
	  /*
	  FileIODialog thankDialog = new FileIODialog(Dialog.findFirstFrame(applet_DGP), 
						      "End results have been successfully sent to server.\n\nThank you for your participation.\nYou have contributed to the solving of the problem.\n\nCheck out http://studentweb.cs.bham.ac.uk/~fsc/summerproject.html regularly to find out whether the problem has been solved\nand if solved, the list of solutions and the names of people who have solved them.\nIn the mean time, keep on participating in the search for a solution to the problem.", 
						      "Thank You") ;
						      */
	  if (suddenEnd) {
	    thankMsg = "This run has been terminated prematurely.\nIntermediate results have been successfully sent to server.\n\nThank you for your participation.\n\nDo check out http://studentweb.cs.bham.ac.uk/~fsc/DGPresults.html regularly\nto find out whether the problem has been solved and if solved, the list of solutions and\nthe names of people who have solved them.\n\nIn the mean time, keep on participating in the search for a solution to the problem.";
	  }
	  else {
	    thankMsg = "End results have been successfully sent to server.\n\nThank you for participating and contributing to the solution of the problem.\n\nDo check out http://studentweb.cs.bham.ac.uk/~fsc/DGPresults.html regularly\nto find out whether the problem has been solved and if solved, the list of solutions and\nthe names of people who have solved them.\n\nIn the mean time, keep on participating in the search for a solution to the problem.";
	  }

	  ThankDialog thankDialog = new ThankDialog(Dialog.findFirstFrame(applet_DGP), thankMsg, "Thank You") ; 

	  thankDialog.show();
	  thankDialog.pack();
	}
       
      }
      catch(MalformedURLException m) {
	System.out.println("MalformedURLException at putObject() !");
	NoConnection = false;
      } 
      catch (OptionalDataException o) {
	System.out.println("OptionalDataException at putObject() !");
	NoConnection = false;
      } 
      catch (ClassNotFoundException c) {
	System.out.println("ClassNotFoundException at putObject() !");
	NoConnection = false;
      } 
      catch(IOException e) {
	System.out.println("IOException when sending end results to server : " + e);
	notifyObservers(new GPMessageStatus("Cannot send end results to server.\nModem may NOT be ON."));
	ModemDialog n = new ModemDialog(Dialog.findFirstFrame(applet_DGP),"Could not connect to Server to send end results !\n\nPlease check that your modem is powered on and hit the <OK> button.\n\nTo exit without sending end results to server, hit the <Cancel> button.", "Network Problem");
	n.show();
	n.pack();
	if (! n.getDialogAns()) {
	  NoConnection = false;
	  notifyObservers(new GPMessageStatus("End results not sent.\nModem may NOT be ON.\nSent abandoned."));
	}
      } //catch
    } //while loop	      

    //*Process the sendStatus string and output the appropriate dialog message to thank user.
    



  }



	/**
	* Uses a quicksort to sort the population destructively
	* into descending order of normalized fitness.
	*/
	private void sort(int low, int high) {
		int index1, index2;
		double pivot;
		Individual temp;

		index1 = low;
		index2 = high;
		pivot = population[(low + high) / 2].normalizedFitness;
		do {
			while (population[index1].normalizedFitness > pivot) {
				index1++;
			}
			while (population[index2].normalizedFitness < pivot) {
				index2--;
			}
			if (index1 <= index2) {
				temp = population[index2];
				population[index2] = population[index1];
				population[index1] = temp;
				index1++;
				index2--;
			}
		} while (index1 <= index2);
		if (low < index2) {
			sort(low, index2);
		}
		if (index1 < high) {
			sort(index1, high);
		}
	}
		
	/**
	* Sorts the population according to normalized fitness. 
	* The population array is destructively modified.
	*/
	void sortPopulationByFitness() {
		sort(0, population.length - 1);
	}

	/**
	* This function has to evaluate the fitness for a specific individual.
	* As a side-effect, it notfies any observers of the GP that an individual
	* has been evaluated and supplies the test cases.
	*/
        /*
	void fitnessFunction(Individual ind, int individualNr) {
		double rawFitness = 0.0;
		ind.hits = 0;
		RealPoint[] testCases = new RealPoint[fitnessCases.get().length];
		for (int index = 0; index < fitnessCases.get().length; index++) {
			double x = fitnessCases.get()[index].x;
			double valueFromProgram = ind.program.eval(x);
			double difference = Math.abs(valueFromProgram - fitnessCases.get()[index].y);
			rawFitness = rawFitness + difference;
			if (difference < 0.01) {
				ind.hits++;
			}
			testCases[index] = new RealPoint(x, valueFromProgram);
		}
		ind.standardizedFitness = rawFitness;
		notifyObservers(new GPMessageEvaluatingIndividual(
				currentGeneration, individualNr, testCases));
	}
	*/

	/**
	* @returns <code>true</code> if the GP algorithm should terminate
	*/
       
	boolean terminationPredicate() {
	  //return currentGeneration >= MAX_GENERATIONS;
	  //return currentGeneration >= maxGenNum.intValue();

	  //if (currentGeneration >= maxGenNum.intValue() || problem.maxHits) {
	  if (currentGeneration > maxGenNum.intValue() || problem.getMaxHits()) {
	    return true;
	  }
	  return false;
	}

	/**
	* Breeds a new population from an old one, thus stepping from one 
	* generation to the next.
	*/
	void evolve() {

	        canSendResult = false;

	        Individual bestOfGeneration;
		
		//double totalAdjFitness = 0.0;
		//double avgAdjFitness = 0.0;
		//Individual[] IndArray = new Individual[numToExport.intValue()];

		if (currentGeneration > 0) {
		  System.out.println("\n*********************************************");

		  currentTime = new Date(); 
		  System.out.println("\nDate is : " + currentTime.getDate());
		  System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 

		  System.out.println("Accepting new imigrants from imigrant queue ...");
		  acceptImigrant();

		  currentTime = new Date(); 
		  System.out.println("\nDate is : " + currentTime.getDate());
		  System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 

		  System.out.println("\nBreeding new population ... ");
		  breedNewPopulation();
		  System.out.println("Population for generation " + currentGeneration + " successfully bred.");

		  currentTime = new Date(); 
		  System.out.println("\nDate is : " + currentTime.getDate());
		  System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 

		}
		System.out.println("Initialising Population fitnesses ... ");
		zeroizeFitnessMeasuresOfPopulation();
		
		currentTime = new Date(); 
		System.out.println("\nDate is : " + currentTime.getDate());
		System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 

		System.out.println("Evaluating fitness of population ... ");
		evaluateFitnessOfPopulation();

		currentTime = new Date(); 
		System.out.println("\nDate is : " + currentTime.getDate());
		System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 

		System.out.println("Normalising fitness of population ... ");
		problem.normalizeFitnessOfPopulation(population);
		//normalizeFitnessOfPopulation();

		currentTime = new Date(); 
		System.out.println("\nDate is : " + currentTime.getDate());
		System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 

		if (solutionFound) {
		  solution = population[individualNrOfSolution];
		}
		  
		//	Sort the population so that the roulette wheel selection is easy:
		System.out.println("Sorting population by fitness in descending order.");
		sortPopulationByFitness();

		currentTime = new Date(); 
		System.out.println("\nDate is : " + currentTime.getDate());
		System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 


		//	Keep track of best-of-run individual:
		
		bestOfGeneration = population[0];
		
		if (((ElitismSelection)elitismChosen.getSelection()).getValue() == ElitismSelection.ELITISM_USED) {
		  if (bestOfRunIndividual == null || bestOfRunIndividual.standardizedFitness > bestOfGeneration.standardizedFitness) {
		    bestOfRunIndividual = bestOfGeneration.copy();
		    generationOfBestOfRunIndividual = currentGeneration;
		  }
		}
		  
		canSendResult = true;
		//System.out.println("\nBest individual of Generation " + currentGeneration + " is\n" + population[0].program.toString(0));
		System.out.println("\nBest of Generation Individual's data : ");
		System.out.println("Its standardised fitness is " + population[0].standardizedFitness);
		System.out.println("Its adjusted fitness is " + population[0].adjustedFitness);
		System.out.println("Its normalised fitness is " + population[0].normalizedFitness);
		System.out.println("Its hits count is " + population[0].hits);

		currentTime = new Date(); 
		System.out.println("\nDate is : " + currentTime.getDate());
		System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 


		double[] fitness = problem.getFitnessesOfAllIndividuals(population);
		notifyObservers(new GPMessageEvaluatingPopulation(currentGeneration, fitness));

			
			//RealPoint[] testCases = new RealPoint[fitnessCases.get().length];
			//RealPoint[] testCases = problem.fitnessCurve(bestOfRunIndividual,this);
		       
			/*
			System.out.println("Best of Run Individual is\n" + bestOfRunIndividual.program.toString(0));
			System.out.println("Its standardised fitness is " + bestOfRunIndividual.standardizedFitness);
			System.out.println("Its adjusted fitness is " + bestOfRunIndividual.adjustedFitness);
			System.out.println("Its normalised fitness is " + bestOfRunIndividual.normalizedFitness);
			System.out.println("Its hits count is " + bestOfRunIndividual.hits);
			*/
			/*
			notifyObservers(new GPMessageBestFound(currentGeneration,
					bestOfRunIndividual.program.toString(0),
					testCases,
					bestOfRunIndividual.adjustedFitness));
					*/
			//}

		//double[] fitness = problem.getFitnessesOfAllIndividuals(population);
		/*
		double[] fitness = new double[population.length];
		for (int i = 0; i < fitness.length; i++) {
		  fitness[i] = population[i].adjustedFitness;
		  //totalAdjFitness = totalAdjFitness + population[i].adjustedFitness;  
		}
		*/
		/*
		avgAdjFitness = sumOfAdjustedFitnesses / population.length;
		System.out.println("\nAverage adjusted fitness of population for generation " + currentGeneration + " is : "  + avgAdjFitness);
		*/

		//

		/* Elitism is used in the program originally. Thus the following method to evaluate the best of run 
		   individual and to plot its graph and display its program. To replicate Koza's experiments which do
		   not use elitism, the best of generation individual is passed into the method instead of the best of
		   run individual. The method is not renamed to evalBestOfGenIndividual as I intend to compare performance
		   with and without elitism in the later part of the project, after I have confirmed that my experiments 
		   are exact replica of those performed by Koza. I am replicating the experiments to provide a basis 
		   of comparison for the performance of my proposed distributed GP system.
		   The evalBestOfRunIndividual method is problem specific at this point in time because the 
		   Symbolic Regression problem plots the evaluation results as a graph whereas the Ant problems do not
		   yet have a GUI and would not share the same GUI as the regression problem as the GUI would have to 
		   consist of a trail with food instead of graphs on a X-Y cooridnate system. */

		//problem.evalBestOfRunIndividual(bestOfRunIndividual,this);
		if (((ElitismSelection)elitismChosen.getSelection()).getValue() == ElitismSelection.ELITISM_USED) {
		  problem.evalBestOfRunIndividual(bestOfRunIndividual,this);
		}
		else {
		  problem.evalBestOfRunIndividual(bestOfGeneration,this);
		}

		// The following are commented out because elitism is not used according to Koza's bk.
		/*
		if (bestOfRunIndividual == null ||
		    //bestOfRunIndividual.standardizedFitness >
		    bestOfRunIndividual.normalizedFitness <
		    //bestOfGeneration.standardizedFitness) {
		    bestOfGeneration.normalizedFitness) {
			bestOfRunIndividual = bestOfGeneration.copy();
			generationOfBestOfRunIndividual = currentGeneration;
			//bestChg = true;
			System.out.println("\n***** Best of Run individual overtaken by best individual from generation " + currentGeneration + " *****");
			*/
			/*
			RealPoint[] testCases = new RealPoint[fitnessCases.get().length];
			for (int i = 0; i < fitnessCases.get().length; i++) {
				double x = fitnessCases.get()[i].x;
				double y = bestOfRunIndividual.program.eval(x);
				testCases[i] = new RealPoint(x, y);
			}
			*/
			
		//problem.evalBestOfRunIndividual(bestOfRunIndividual,this);
			
			//RealPoint[] testCases = new RealPoint[fitnessCases.get().length];
			//RealPoint[] testCases = problem.fitnessCurve(bestOfRunIndividual,this);
		       
			/*
			System.out.println("Best of Run Individual is\n" + bestOfRunIndividual.program.toString(0));
			System.out.println("Its standardised fitness is " + bestOfRunIndividual.standardizedFitness);
			System.out.println("Its adjusted fitness is " + bestOfRunIndividual.adjustedFitness);
			System.out.println("Its normalised fitness is " + bestOfRunIndividual.normalizedFitness);
			System.out.println("Its hits count is " + bestOfRunIndividual.hits);
			*/
			/*
			notifyObservers(new GPMessageBestFound(currentGeneration,
					bestOfRunIndividual.program.toString(0),
					testCases,
					bestOfRunIndividual.adjustedFitness));
					*/
			//}

		//double[] fitness = problem.getFitnessesOfAllIndividuals(population);
		/*
		double[] fitness = new double[population.length];
		for (int i = 0; i < fitness.length; i++) {
		  fitness[i] = population[i].adjustedFitness;
		  //totalAdjFitness = totalAdjFitness + population[i].adjustedFitness;  
		}
		*/
		/*
		avgAdjFitness = sumOfAdjustedFitnesses / population.length;
		System.out.println("\nAverage adjusted fitness of population for generation " + currentGeneration + " is : "  + avgAdjFitness);
		*/

		//notifyObservers(new GPMessageEvaluatingPopulation(currentGeneration, fitness));
 		

		if (counter >= exportGenNum.intValue()) {
		  // if (bestChg && counter >= exportGen) {
		  //bestChg = false ;
		  
		  System.out.println("\n%%%%% Selecting " + numToExport.intValue() + " individual(s) for export to other hosts ... %%%%%");
		  System.out.println("Creating array for individuals to be exported ...");
		  
		  if (SEED != applet_DGP.defaultSEED) {
		    createAndExportIndivArray(numToExport.intValue());
		  }
		  /*
		  //exportIndividual(bestOfRunIndividual);
	
		  Individual[] IndArray = new Individual[numToExport.intValue()];
		  counter = 0;
		  int index = 0;

		  for (int indCount = 0; indCount < numToExport.intValue(); indCount++) {

		    System.out.println("Looking for the best individual(s) that has not been sent");
		  //System.out.println("Sent flag of population[" + index + "] : " + population[index].sent);

		  while (population[index].sent) {
		    //System.out.println("population[" + index + "].sent is true. Inside loop to search for next unsend individual.");
		    index ++;
		  }

		  // set sent to true before exporting so that individual cannot be resent by this W.S as well as other W.S.
		  population[index].sent = true;

		  System.out.println("Population[" + index + "] was selected for export and its 'send' flag is set to true.");

		  //System.out.println("Sent flag of population[" + index + "] : " + population[index].sent);

		  IndArray[indCount] = population[index];

		  System.out.println("Population[" + index + "] was inserted into individual array.");
		  }

		  System.out.println("Creation of Individual Array completed.");
		  exportIndividual(IndArray);
		  */
		  
		}	
		
		/* following code and variables are needed only when running DGPExp for experiments. */
		/*
		if ((genToStartAnotherPop > 0) && (genToStartAnotherPop == currentGeneration)) {
		  if (remotePop == true) {
		    //DGPExpRemote.informToStart();
		    
		    try {
		      DGPNotifyRMI n = (DGPNotifyRMI)Naming.lookup("rmi://" + DGPExpRemote.fromHost + "/DGP-Inform-Start");
		      String status = n.notifyStart(thisHost);
		      System.out.println(status);
		    }
		    catch (Exception e) {
		      System.out.println("Exception in informToStart : " + e);    
		    }

		  }
		  else {
		    DGPExp.DGPExpObject.notifyToStartNewPop();
		    */
		    /*
		    Thread nThread = new NotifyStartThread();
		    nThread.start();
		    */
		/*  }
		}
		*/

		counter ++;

		currentGeneration++;
	}

	/**
	* @return an individual in the specified population whose
	* normalized fitness is greater than the specified value.
	*/
	Individual findFitnessProportionateIndividual(double afterThisFitness) {
		//	All we need to do is count along the population from the
		//	beginning adding up the fitness until we get past the
		//	specified point:
		int indexOfSelectedIndividual;
		double sumOfFitness = 0.0;
		int index = 0;

		//System.out.println("random no. for fitnessProportionateSelection : " + afterThisFitness);
		
		while (index < population.length && sumOfFitness < afterThisFitness) {
			//	Sum up the fitness values.
			sumOfFitness = sumOfFitness + population[index].normalizedFitness; 
			index++;
		}
		if (index >= population.length) {
			indexOfSelectedIndividual = population.length - 1;
		} else {
			indexOfSelectedIndividual = index - 1;
		}
		
		//System.out.println("index of individual selected is : " + indexOfSelectedIndividual);

		return population[indexOfSelectedIndividual];
	}

	/**
	* @return picks some individuals from the population at random and
	* returns the best one.
	*/
	Individual findIndividualUsingTournamentSelection() {
		int TournamentSize = Math.min(population.length, tournamentSize.intValue());

		Hashtable table = new Hashtable();
		while (table.size() < TournamentSize) {
			int key = random.nextInt(population.length);
			table.put(new Integer(key), population[key]);
		}
		Enumeration e = table.elements();
		Individual best	= (Individual)e.nextElement();
		double bestFitness = best.standardizedFitness;
		while (e.hasMoreElements()) {
			Individual individual = (Individual)e.nextElement();
			double thisFitness = individual.standardizedFitness;
			if (thisFitness < bestFitness) {
	  		best = individual;
				bestFitness = thisFitness;
			}
		}
		return best;
	}

	/**
	* @return finds an individual in the population according to the
	* defined selection method.
	*/
	Individual findIndividual() {
		Individual ind = null;
		switch (((MethodOfSelection)methodOfSelection.getSelection()).getValue()) {
			case MethodOfSelection.TOURNAMENT:
				ind = findIndividualUsingTournamentSelection();
				break;
			case MethodOfSelection.FITNESS_PROPORTIONATE:
				ind = findFitnessProportionateIndividual(random.nextDouble());
				break;
		}
		return ind;
	}

	/**
	* @return some initial fitness cases, in text form, as pairs of
	*		x-y values
	*/
  /*
	String makeDefaultFitnessCases() {
		int NrOfFitnessCases = 20;
		String s = "// 3*x^4 - 3*x + 1\n";
		for (int i = 0; i < NrOfFitnessCases; i++) {
		        double x = (double)i / (NrOfFitnessCases - 1);
		        //int x = random.nextInt(100); 

			if (x == 0.0) {
				x = 1.0e-7;
			}

			//double y = 1.0 + (-3.0)*x + (3.0)*x*x*x*x;
			//double y = 1.0 + x*x*x* + (- x*x);
			//double y = x*x + (-1.8)*x + 0.7;
			//double y = Math.cos(Math.cos(x*x + x + (-1.0)));
			//double y = x*x + (-1.8)*x + 0.7 / Math.cos(Math.cos(x*x + x + (-1.0)));
			double y = 0.2 + x * Math.exp(x) + (-3.0)*x*x*x / Math.cos(Math.cos(Math.exp(x) + x + (-1.0)));
			//double y = (5.0)*x*x*x + (- x*x*x*x) + (2.0);

			//	The stream tokenizer does not handle numbers with an exponent.
			//	Therefore, clamp small numbers like 1.2e-6 to 0: 
			if (Math.abs(x) < 1.0e-5) {
				x = 0;
			};
			if (Math.abs(y) < 1.0e-5) {
				y = 0;
			};
			s = s + x + "  " + y + "\n";
		}
		return s;
	}
	*/

	/**
	* This is the body of the GP thread. It will be called by the
	* thread's start() method.
	*/
	public void run() {
	  int choiceNo;
		//	Prime the random generator with the same seed for each run,
		//	so that we get reproducible results:

	  /*
	  System.setSecurityManager(new RMISecurityManager());
	  try {
	    ImportImigrant h = new ImportImigrant();
	    Naming.rebind("importImigrant",h);
	    System.out.println("ImportImigrant server ready");
	  }
	  catch (RemoteException re) {
	    System.out.println("Exception in GP.run() : " + re);
	  }
	  catch (MalformedURLException e) {
	    System.out.println("MalformedURLException in GP.run(): " + e);
	  }
	  */

	  if ((applet_DGP.Application) && (fileName != null)) {
	    popFile = new File(fileName);
	    if (popFile.exists()) {
	      System.out.println("Setting PopFileFound to true ");
	      PopFileFound = true; 
	    } else {
	      PopFileFound = false;
	    }
	  } else {
	    PopFileFound = false;
	  }

	  //if ((! applet_DGP.Application && runComplete) || (applet_DGP.Application && ! PopFileFound && ! firstRun)) {
	  if ((! applet_DGP.Application) || (applet_DGP.Application && ! PopFileFound)) {
	    //get seed from server for runs other than the first one after start-up.
	    System.out.println("Requesting new Seed from server to create population");
	    
	    /*
	    if (! applet_DGP.Application) { 
		String appletURL = "//" + applet_DGP.getCodeBase().getHost() + "/importImigrant";
		System.out.println("appletURL : " + appletURL); 
		//i = (imigrant)Naming.lookup("//" + applet_DGP.getCodeBase().getHost() + "/importImigrant");
		i = (imigrant)Naming.lookup("//" + "hamm.cs.bham.ac.uk" + "/importImigrant");
	      }
	      else { 
		i = (imigrant)Naming.lookup("rmi://" + "laa-laa.cs.bham.ac.uk" + "/importImigrant");
	      }
	      //imigrant i = (imigrant)Naming.lookup("rmi://" + line + "/importImigrant");
	      //String status = i.receiveImigrant(goodIndividual,thisHost);
	      */
		  if (DGPClient.experiment) {
		      servletURL = servletHost + "SeedExpServlet?ProbName=" + applet_DGP.probName ;
		  }
		  else {
		      servletURL = servletHost + "Seed24Servlet?ProbName=" + applet_DGP.probName ;
		  }
	    //servletURL = servletHost + "Seed15Servlet?ProbName=" + "LosAltosHills" ;
	    //servletURL = servletHost + "Seed3Servlet";
	    //@@servletURL = servletHost + "Hello9Servlet";
	    System.out.println("servletURL : " + servletURL);
	    
	    Hashtable pTable = new Hashtable();
	    parmTable = createParmTable(pTable);
	    if (parmTable == null) {
	      System.out.println("parmTable is null !!!");
	    }
	    else {
	      System.out.println("parmTable is not null --- successfully created");
	    }
	      
	    Hashtable replyObj = null;

	    
	    NoConnection = true;
	    while (NoConnection) {
	      notifyObservers(new GPMessageStatus("Sending GP Settings to server ..."));
	      System.out.println("Sending GP settings to server ... ");
	      try {
		System.out.println("going to run putObject for parmTable");
		//putObject(servletURL, parmTable);
		replyObj = (Hashtable)putObject(servletURL, parmTable);
		NoConnection = false;
		notifyObservers(new GPMessageStatus("GP Settings sent to server & seed received."));
		System.out.println("*** GP Settings successfully sent to server & seed received ! ***");
	      }
	      catch(MalformedURLException m) {
		System.out.println("MalformedURLException at putObject() !");
		NoConnection = false;
	      } 
	      catch (OptionalDataException o) {
		System.out.println("OptionalDataException at getObject() !");
		NoConnection = false;
	      } 
	      catch (ClassNotFoundException c) {
		System.out.println("ClassNotFoundException at getObject() !");
		NoConnection = false;
	      } 
	      catch(IOException e) {
		System.out.println("IOException at putObject() !");
		notifyObservers(new GPMessageStatus("Cannot send GP settings to server.\nModem may NOT be ON !"));
		ModemDialog n = new ModemDialog(Dialog.findFirstFrame(applet_DGP),"Could not connect to Server to send GP settings !\nPlease check that your modem is powered on.", "Network Problem");
		n.show();
		n.pack();
		if (! n.getDialogAns()) {
		  NoConnection = false;
		  notifyObservers(new GPMessageStatus("Abandon the sending of GP settings to server."));
		  System.out.println("Abandon the sending of GP settings to server ");
		}
	      }
	      
	    }
	    
	    /*
	    NoConnection = true;
	    while (NoConnection) {
	      notifyObservers(new GPMessageStatus("Obtaining seed from server ..."));
	      System.out.println("Obtaining seed from server ...");
	      try {
		//replyObj = (Hashtable)getObject(servletURL);
		replyObj = (Hashtable)getObject(servletURL, parmTable);
		NoConnection = false;
		//notifyObservers(new GPMessageStatus("Seed obtained from server\nis " + SEED));
		notifyObservers(new GPMessageStatus("Seed obtained from server"));
		System.out.println("Send obtained from server");
	      }
	      catch (MalformedURLException m) {
		System.out.println("MalformedURLException at getObject() !");
		NoConnection = false;
	      }
	      catch (OptionalDataException o) {
		System.out.println("OptionalDataException at getObject() !");
		NoConnection = false;
	      } 
	      catch (ClassNotFoundException c) {
		System.out.println("ClassNotFoundException at getObject() !");
		NoConnection = false;
	      }
	      catch (IOException e) {
		System.out.println("IOException at getObject() !");
		notifyObservers(new GPMessageStatus("Cannot obtain seed from Server.\nModem may NOT be ON !"));
		ModemDialog n = new ModemDialog(Dialog.findFirstFrame(applet_DGP),"Could not connect to Server to retrieve Seed !\nPlease check that your modem is powered on.", "Network Problem");
		n.show();
		n.pack();
		if (! n.getDialogAns()) {
		  NoConnection = false;
		  notifyObservers(new GPMessageStatus("Abandon obtaining seed from server.\nDefault seed is used instead."));
		}
	      }
	    }
	    */
	    
	    Integer recvSeedObj;
	    int recvSeed;
	    String recvID;
	    if (replyObj != null) {
	      System.out.println("replyObj is not null -- processing replyObj");
	      //recvSeed = ((Integer)replyObj.get("SeedObj")).intValue();
	      recvSeedObj = (Integer)replyObj.get("SeedObj");
		if (recvSeedObj != null) {
		  SEED = recvSeedObj.intValue();
		}
		recvID = (String)replyObj.get("ID");
		if (recvID !=  null) {
		  clientID = recvID;
		}
		errorMsg = (String)replyObj.get("ErrorMsg");
		if ((errorMsg.trim()).equals("NoSuchProblem")) {
		  System.out.println("This problem is no longer handled by the servlet.");
		  notifyObservers(new GPMessageStatus("This problem is no longer\nhandled by the servlet."));
		  ModemDialog n = new ModemDialog(Dialog.findFirstFrame(applet_DGP),"*** The problem you are trying to solve is no longer handled by the servlet. ***\n\nPlease download the new applet or application to solve the new problem from http://studentweb.cs.bham.ac.uk/~fsc/DGP.html. Alternatively, you could still run the applet or application in a standalone mode without communicating with the servlet.\n\nPlease press the <OK> button to continue the run in a standalone mode or the <Cancel> button to end the run and download the new applet and application.", "Problem no longer handled");
		  n.show();
		  n.pack();
		  if (! n.getDialogAns()) {
		    notifyObservers(new GPMessageStatus("Current problem is not being\nsolved by servlet.\n\nRun ending ..."));
		    stop();
		  }
		  else {
		    SEED = applet_DGP.defaultSEED ;
		  }
		}
	    }
	    
	    /*
	    NoConnection = true;
	    while (NoConnection) {
		notifyObservers(new GPMessageStatus("Obtaining seed from server ..."));
	    try {
		if (applet_DGP.i == null) {
		  System.out.println("remote object is null. Naming.lookup for remote object");
		  if (! applet_DGP.Application) {
		    String appletURL = "//" + applet_DGP.getCodeBase().getHost() + "/importImigrant";
		    System.out.println("appletURL : " + appletURL); 
		    //i = (imigrant)Naming.lookup("//" + applet_DGP.getCodeBase().getHost() + "/importImigrant");
		    applet_DGP.i = (imigrant)Naming.lookup("//" + "hamm.cs.bham.ac.uk" + "/importImigrant");
		  }
		  else { 
		    applet_DGP.i = (imigrant)Naming.lookup("rmi://" + "hamm.cs.bham.ac.uk" + "/importImigrant");
		  }
		}
		
		System.out.println("Obtaining seed from server ... ");
		SEED = applet_DGP.i.getSeed();
		System.out.println("Seed obtained from server is " + SEED);
		
		NoConnection = false;
		notifyObservers(new GPMessageStatus("Seed obtained from server\nis " + SEED));
	    }
		
	      catch (Exception e) {
	      //catch (RemoteException e) {
		System.out.println("Exception when obtaining seed: " + e);
		notifyObservers(new GPMessageStatus("Cannot obtain seed from Server.\nModem may NOT be ON !"));
		ModemDialog n = new ModemDialog(Dialog.findFirstFrame(applet_DGP),"Could not connect to Server to retrieve Seed !\nPlease check that your modem is powered on.", "Network Problem");
		n.show();
		n.pack();
		if (! n.getDialogAns()) {
		  NoConnection = false;
		  notifyObservers(new GPMessageStatus("Abandon obtaining seed from server.\nDefault seed is used instead."));
		}
	      }
  	    }
	    */
	  }
	  else 
	    if ((applet_DGP.Application) && (PopFileFound)) {	      
	      loadGPSettings(fileName);
	    }
	  
	  runComplete = false;
	  //firstRun = false;
	   
	  random.setSeed(SEED);
	  System.out.println("New seed set for random number generator");


	  System.out.println("\nHost Name : " + thisHost);
	  System.out.println("Problem Name : " + problem.getProblemName());
	  System.out.println("Client ID : " + clientID);
	  System.out.println("Error Msg from servlet : " + errorMsg);
	  System.out.println("\nParameter Settings :");
	  System.out.println("Randomiser seed : " + SEED);
	  System.out.println("Population size : " + populationSize.intValue());
	  System.out.println("Max number of generations in a run : " + maxGenNum.intValue());
	  System.out.println("Period (no. of generations) to export individuals : " + exportGenNum.intValue());
	  System.out.println("Number of individuals to export : " + numToExport.intValue());
	  //System.out.println("Size of imigrant queues : " + imigrantQueueSize.intValue());
	  System.out.println("Number of individuals to import : " + imigrantQueueSize.intValue());
	  System.out.println("Max depth for new individuals : " + maxDepthForNewIndividuals.intValue());
	  System.out.println("Max depth for individuals after crossover : " + maxDepthForIndividualsAfterCrossover.intValue());
	  System.out.println("Max depth for new subtrees in mutants : " + maxDepthForNewSubtreesInMutants.intValue());
	  System.out.println("Probability of Crossover : " + crossoverFraction.doubleValue());
	  System.out.println("Probability of Reproduction : " + fitnessProportionateReproFraction.doubleValue());
	  System.out.println("Probability of Mutation : " + mutationFraction.doubleValue());
	  //System.out.println("Number of fitness cases : " + problem.fitnessCases.get().length);
	  //System.out.println("Number of fitness cases : " + problem.NrOfFitnessCases);
	  System.out.println("Number of fitness cases : " + problem.getNrOfFitnessCases());
	  //System.out.println("Fitness Cases :\n" + problem.fitnessCaseString);
	  System.out.println("Fitness Cases :\n" + problem.getFitnessCaseString());
	  System.out.println("Method of Generation : " + ((MethodOfGeneration)methodOfGeneration.choices[methodOfGeneration.selection]).toString());
	  System.out.println("Method of Selection : " + ((MethodOfSelection)methodOfSelection.choices[methodOfSelection.selection]).toString());
	  System.out.println("Elitism : " + ((ElitismSelection)elitismChosen.choices[elitismChosen.selection]).toString());
	  System.out.println("Tournament selection group size : " + tournamentSize.intValue());
	  
	  System.out.print("\nFunction Set : ");
	  for (int i=0; i < functionSet.selections.length; i++) {
	    choiceNo = functionSet.selections[i];
	    System.out.print(functionSet.choices[choiceNo].toString() + ";  ");
	  }

	  System.out.print("\nTerminal Set : ");
	  for (int i=0; i < terminalSet.selections.length; i++) {
	    choiceNo = terminalSet.selections[i];
	    System.out.print(terminalSet.choices[choiceNo].toString() + ";  ");
	  }
	  
		//notifyObservers(new GPMessageFitnessCasesSet(problem.fitnessCases.get()));
		notifyObservers(new GPMessageFitnessCasesSet(problem.getFitnessCases().get()));

		generationOfBestOfRunIndividual = 0;
		bestOfRunIndividual = null;

		currentTime = new Date(); 
		System.out.println("\nDate is : " + currentTime.getDate());
		System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 
		
		if ((PopFileFound) && (applet_DGP.Application)) { 
		  System.out.println("\nLoading saved population from file " + fileName);
		  notifyObservers(new GPMessageStatus("Loading from file\n" + fileName + " ..."));
		  loadPopulation(fileName);
		  //applet_DGP.settingsButton.enable(false);
		} else {		
                System.out.println("\nCreating initial population ... ");
		notifyObservers(new GPMessageStatus("Creating initial population ..."));
		createPopulation();
		}
	       		
		currentGeneration = 0;
		problem.initMaxHits();
                System.out.println("Initial population (Generation " + currentGeneration + ") is created/loaded.");

		currentTime = new Date(); 
		System.out.println("\nDate is : " + currentTime.getDate());
		System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 

		counter = 1;
		//bestChg = false;

		System.out.println("currentGeneration : " + currentGeneration);
		System.out.println("maxHits : " + problem.getMaxHits());

	
		while (! terminationPredicate()) {
			try {
				evolve();
				thread.sleep(1);
				//	thread.sleep() or thread.yield()? 
				//	I'm not sure about this. The JDK tutorial suggests yielding
				//	so that other threads get a chance to run on host systems
				//	without	built-in thread support. In my experience with
				//	Windows 95 and NT, user interface response got much worse
				//	with yield()ing. I have to wait for feedback from users on
				//	other platforms to get a clue.
			} catch(Exception e) {
				crash();
			}
		}
		//	stop() will notify any Observers that the GP has come to an end:
		

		//if (applet_DGP.drawChoice.getSelectedIndex() == 1) {
		//drawBest = true;
		//problem.evalBestOfRunIndividual(population[0],this);
		//}
		
		System.out.println("\n~~~~~~~~~~~~~~~~~~~~~~ End of Run ~~~~~~~~~~~~~~~~~");

		currentTime = new Date(); 
		System.out.println("\nDate is : " + currentTime.getDate());
		System.out.println("Time is : " + currentTime.getHours() + "(Hrs) " + currentTime.getMinutes() + "(Mins) " + currentTime.getSeconds() + "(Secs)\n"); 
		
		if (solution == null) {
		  System.out.println("NO SOLUTION FOUND ! ");
		  notifyObservers(new GPMessageStatus("No solution found in this run."));
		}
		else {
		  //if (! (solution == null)) {
		//if (solution == null) {
		  System.out.println("SOLUTION FOUND ! ");
		  genNrOfSolution = currentGeneration - 1;
		  notifyObservers(new GPMessageStatus("Solution found in\ngeneration " + genNrOfSolution + "\n of this run."));
		  //if (sendSol) {
		    System.out.println("Sending solution to and setting generation of solution for all workstations.");
		    //solutionFirstFoundGenNr = currentGeneration - 1;

		    //***solGenNrVector.addElement(new Integer(genNrOfSolution));

		    //solGenNrTable.put(thisHost, new Integer(genNrOfSolution));
		   
		    //***toSetGenOfSolution = true;

		    //Individual[] solutionArray = {solution};
		    
		    NameDialog n = new NameDialog(Dialog.findFirstFrame(applet_DGP),
						  "*** Congratulations ! You have found a solution to the problem. ***\n\nThe solutions found and information about participants\nwho have solved the problem are announced in\nhttp://studentweb.cs.bham.ac.uk/~fsc/DGPresults.html.\n\nPlease enter your name, organisation / university / school and\ndesignation (eg. programmer, Phd student)\nin the following fields and hit the <Set> button.\nTo exit without providing any information, hit the <Cancel> button.", 
						  "Name", "Organisation", "Designation",
						  "Solution Found");
		    n.show();
		    n.pack();
		    
		    if (n.getName() != null) {
		      solName = n.getName();
		    }
		    else {
		      solName = "";
		    }
		    if (n.getOrg() != null) {
		      solOrg = n.getOrg();
		    }
		    else {
		      solOrg = "";
		    }
		    if (n.getDesignation() != null) {
		      solDesignation = n.getDesignation();
		    }
		    else {
		      solDesignation = "";
		    }
		    
		}
		//send results regardless of whether solution is found. Send only if this indv has not just been exported.
		runComplete = true;
		/*
		if (SEED != applet_DGP.defaultSEED) {
		  if (! ((counter == 1) && (lastSendSucceed))) { 
		    Individual[] solutionArray = {population[0]}; 
		    exportIndividual(solutionArray);
		  }
		}
		*/

		if (SEED != applet_DGP.defaultSEED) {
		  sendEndResults();
		}


		    //}

		  /*
		  System.out.println("\nSolution is \n" + solution.program.toString(0));
		  System.out.println("\nGeneration of solution is " + genNrOfSolution);
		  System.out.println("Generation of first solution found among distributed workstations is " + solutionFirstFoundGenNr);
		  System.out.println("\nIts standardised fitness is " + solution.standardizedFitness);
		  System.out.println("Its adjusted fitness is " + solution.adjustedFitness);
		  System.out.println("Its normalised fitness is " + solution.normalizedFitness);
		  System.out.println("Its hits count is " + solution.hits + "\n\n");
		  */ 

		if (((ElitismSelection)elitismChosen.getSelection()).getValue() == ElitismSelection.ELITISM_USED) {
		  System.out.println("\nBest of Run Individual is \n" + bestOfRunIndividual.program.toString(0));
		  System.out.println("Generation of best of run individual is " + generationOfBestOfRunIndividual);  
		  System.out.println("Its standardised fitness is " + bestOfRunIndividual.standardizedFitness);
		  System.out.println("Its adjusted fitness is " + bestOfRunIndividual.adjustedFitness);
		  System.out.println("Its normalised fitness is " + bestOfRunIndividual.normalizedFitness);
		  System.out.println("Its hits count is " + bestOfRunIndividual.hits + "\n\n");
		}
		else {		  
		  System.out.println("\nSolution/Best of run individual is \n" + population[0].program.toString(0));
		  System.out.println("\nGeneration of solution is " + genNrOfSolution);
		  //System.out.println("Generation of first solution found among distributed workstations is " + solutionFirstFoundGenNr);
		  System.out.println("\nIts standardised fitness is " + population[0].standardizedFitness);
		  System.out.println("Its adjusted fitness is " + population[0].adjustedFitness);
		  System.out.println("Its normalised fitness is " + population[0].normalizedFitness);
		  System.out.println("Its hits count is " + population[0].hits + "\n\n");
		}

		/* Commented out because elitism not used here according to Koza's experiments
		else {   
		System.out.println("\Best of Run Individual is \n" + bestOfRunIndividual.program.toString(0));
		System.out.println("Generation of best of run individual is " + generationOfBestOfRunIndividual);  
		System.out.println("Its standardised fitness is " + bestOfRunIndividual.standardizedFitness);
		System.out.println("Its adjusted fitness is " + bestOfRunIndividual.adjustedFitness);
		System.out.println("Its normalised fitness is " + bestOfRunIndividual.normalizedFitness);
		System.out.println("Its hits count is " + bestOfRunIndividual.hits + "\n\n");
		}
		*/

		//firstRun = false;
		
		if (PopFileFound) {
		  popFile.delete();
		  fileName = null;
		}

		drawBest = true;
		if (((ElitismSelection)elitismChosen.getSelection()).getValue() == ElitismSelection.ELITISM_USED) {
		  problem.evalBestOfRunIndividual(bestOfRunIndividual,this);
		}
		else {
		  problem.evalBestOfRunIndividual(population[0],this);
		}
  //runComplete = true;
		  
		stop();	       
		
	}
 

        /**
	* Search for the user-specified or default file containing the saved population and seed.
	*/
        /*
        boolean findPopFile(String fileName) {
	  popFile = new File(fileName);
	  if ( ! popFile.exists()) {
	    return false;
	  } else {
	  return true;
	  }
	}
	*/

	/**
	* @return the index'th subtree satisfying a specific condition,
	* for example the	3rd terminal of a tree.
	* Subtrees are numbered from left to right,
	* depth first. The test function of the condition is applied to all nodes
	* of the tree. It must evaluate to true for the desired kind of subtrees.
	* getSubtree() returns not only a pointer to the specified subtree, but
	* also a pointer to the subtree's parent and and index for its position
	* in the list of the parent's children.
	* If the subtree is not found, the function returns null.
	*/
	TreeHook getSubtree(Program tree, int index, Condition cond) {
		//	Hack:	I need a global counter across the recursive calls to Walk().
		//	In a Wirthian language, a VAR parameter would be fine.
		//	In Java, I misuse a single-element array: 
		int[] count = { index };
		return Walk(tree, count, cond, null, -1);
	}

	/**
	*
	*/
	private TreeHook Walk(Program tree, int[] count, Condition cond, Function parent, int childIndex) {
		if (cond.test(tree) && count[0] == 0) {
			return new TreeHook(tree, parent, childIndex);
		} else {
			TreeHook hook = null;
			if (tree instanceof Function) {
				Function func = (Function)tree;
				for (int a = 0; a < func.arg.length && count[0] > 0; a++) {
					if (cond.test(func.arg[a])) {
						count[0]--;
					}
					hook = Walk(func.arg[a], count, cond, func, a);
				}
			}
			return hook;
		}
	}

	/**
	* Notifies any observers of changes in the GP algorithm's state.
	*	Observers must register with this Observable.
	*/
	public void notifyObservers(Object  arg) {
		//	Why should I call notifyObservers if nothing has changed?
		//	I change the semantics of notifyObservers() here and always
		//	set the 'changed' flag first:
		setChanged();
		super.notifyObservers(arg);
	}



  //createParmTable for sending to servlet when requesting for seed.
  Hashtable createParmTable(Hashtable pTable) {
    int choiceNo;
    
    pTable.put("Host_Name", thisHost);
    pTable.put("Prob_Name", problem.getProblemName());
    pTable.put("Pop_Size", new Integer(populationSize.intValue()));
    pTable.put("Max_Gen", new Integer(maxGenNum.intValue()));
    pTable.put("Export_Interval", new Integer(exportGenNum.intValue()));
    pTable.put("Export_Num", new Integer(numToExport.intValue()));
    pTable.put("Import_Num", new Integer(imigrantQueueSize.intValue()));
    pTable.put("Max_Depth_For_New_Indv", new Integer(maxDepthForNewIndividuals.intValue()));
    pTable.put("Max_Depth_For_Indv_After_Crossover", new Integer(maxDepthForIndividualsAfterCrossover.intValue()));
    pTable.put("Max_Depth_For_New_Subtrees_In_Mutants", new Integer(maxDepthForNewSubtreesInMutants.intValue()));
    pTable.put("Crossover_Prob", new Double(crossoverFraction.doubleValue()));
    pTable.put("Reproduction_Prob", new Double(fitnessProportionateReproFraction.doubleValue()));
    pTable.put("Mutation_Prob", new Double(mutationFraction.doubleValue()));
    pTable.put("Fitness_Cases_Num", new Integer(problem.getNrOfFitnessCases()));
    pTable.put("Fitness_Cases", problem.getFitnessCaseString());
    pTable.put("Method_Of_Generation", ((MethodOfGeneration)methodOfGeneration.choices[methodOfGeneration.selection]).toString());
    pTable.put("Method_Of_Selection", ((MethodOfSelection)methodOfSelection.choices[methodOfSelection.selection]).toString());
    pTable.put("Elitism", ((ElitismSelection)elitismChosen.choices[elitismChosen.selection]).toString());
    pTable.put("Tournament_selection_group_size", new Integer(tournamentSize.intValue()));
     
    pTable.put("Function_Set_length", new Integer(functionSet.selections.length));
    for (int i=0; i < functionSet.selections.length; i++) {
      choiceNo = functionSet.selections[i];
      pTable.put("Function_Set_" + i , functionSet.choices[choiceNo].toString());
    }

    pTable.put("Terminal_Set_length", new Integer(terminalSet.selections.length));
    for (int i=0; i < terminalSet.selections.length; i++) {
      choiceNo = terminalSet.selections[i];
      pTable.put("Terminal_Set_" + i, terminalSet.choices[choiceNo].toString());
    }
    
    return pTable;

  }

  void saveSettingsObj(Hashtable ss) {

    NumberUI newMaxGen = new NumberUI("Max. no. of generations in a run", maxGenNum.intValue() - currentGeneration + 1 , 1, 1000);
    
    ss.put("Pop_Size", populationSize);
    ss.put("Max_Gen", newMaxGen);
    ss.put("Export_Interval", exportGenNum);
    ss.put("Export_Num", numToExport);
    ss.put("Import_Num", imigrantQueueSize);
    ss.put("Max_Depth_For_New_Indv", maxDepthForNewIndividuals);
    ss.put("Max_Depth_For_Indv_After_Crossover", maxDepthForIndividualsAfterCrossover);
    ss.put("Max_Depth_For_New_Subtrees_In_Mutants", maxDepthForNewSubtreesInMutants);
    ss.put("Crossover_Prob", crossoverFraction);
    ss.put("Reproduction_Prob", fitnessProportionateReproFraction);
    ss.put("Mutation_Prob", mutationFraction);
    //ss.put("Fitness_Cases_Num", new Integer(problem.getNrOfFitnessCases()));
    //ss.put("Fitness_Cases", problem.getFitnessCaseString());
    ss.put("Method_Of_Generation", methodOfGeneration);
    ss.put("Method_Of_Selection", methodOfSelection);
    ss.put("Elitism", elitismChosen);
    ss.put("Tournament_selection_group_size", tournamentSize);
    ss.put("Function_Set", functionSet);
    ss.put("Terminal_Set", terminalSet);    

    maxGenNum = 
		  //new NumberUI("Max. no. of generations in a run", 50, 20, 1000);
		  new NumberUI("Max. no. of generations in a run", 50, 1, 1000);

  }


  void restoreSettingsObj(Hashtable rt) {

	populationSize = (NumberUI)rt.get("Pop_Size");
	maxGenNum = (NumberUI)rt.get("Max_Gen");
	exportGenNum = (NumberUI)rt.get("Export_Interval");
	numToExport = (NumberUI)rt.get("Export_Num");
	imigrantQueueSize = (NumberUI)rt.get("Import_Num");
	maxDepthForNewIndividuals = (NumberUI)rt.get("Max_Depth_For_New_Indv");
	maxDepthForIndividualsAfterCrossover = (NumberUI)rt.get("Max_Depth_For_Indv_After_Crossover");
	maxDepthForNewSubtreesInMutants = (NumberUI)rt.get("Max_Depth_For_New_Subtrees_In_Mutants");
	crossoverFraction = (NumberUI)rt.get("Crossover_Prob");
	fitnessProportionateReproFraction = (NumberUI)rt.get("Reproduction_Prob");
	mutationFraction = (NumberUI)rt.get("Mutation_Prob");
	methodOfGeneration = (ChoiceUI)rt.get("Method_Of_Generation");
	methodOfSelection = (ChoiceUI)rt.get("Method_Of_Selection");
	elitismChosen = (ChoiceUI)rt.get("Elitism");
	tournamentSize = (NumberUI)rt.get("Tournament_selection_group_size");
	functionSet = (SetUI)rt.get("Function_Set");
	terminalSet = (SetUI)rt.get("Terminal_Set");
  }




  Object putObject(String servletURL, Serializable obj)
    throws MalformedURLException, IOException, OptionalDataException, ClassNotFoundException {
      
      System.out.println("inside putObject()");

      //HttpURLConnection uc = (HttpURLConnection) new URL(servletURL).openConnection();
      URLConnection uc = new URL(servletURL).openConnection();
      System.out.println("URL Connection opened to " + servletURL);
      //uc.setDoInput(true);
      //uc.setDoOutput(true);
      uc.setUseCaches(false);
      //uc.setRequestMethod("POST");
      uc.setDoOutput(true);
      ObjectOutputStream objOut = new ObjectOutputStream(new GZIPOutputStream(uc.getOutputStream()));
      //ObjectOutputStream objOut = new ObjectOutputStream(new BufferedOutputStream(uc.getOutputStream()));
      System.out.println("Object output stream created");
      objOut.writeObject(obj);
      System.out.println("object written to object output stream.");
      objOut.flush();
      objOut.close();
      System.out.println("object output stream flushed and closed.");
      
      System.out.println("Object sent !");
      InputStream is = uc.getInputStream() ; 

      //ObjectInputStream objIn = new ObjectInputStream(new GZIPInputStream(u.openStream()));
      ObjectInputStream objIn = new ObjectInputStream(new GZIPInputStream(uc.getInputStream()));
      Object rtn = objIn.readObject();
      objIn.close();
      
      System.out.println("Object received !");

      return rtn;



 }
    
  /*
  //Object getObject(String servletURL)
  Object getObject(String servletURL, Serializable obj)
    throws MalformedURLException, IOException, OptionalDataException, ClassNotFoundException {
      
      System.out.println("inside getObject()");
      
      //URLConnection uc = new URL(servletURL).openConnection();
      
      //uc.setDoInput(true);
      //uc.setUseCaches(false);
      
      //ObjectInputStream objIn = new ObjectInputStream(new GZIPInputStream(uc.getInputStream()));
      //Object rtn = objIn.readObject();
      //objIn.close();
      
      
      URL u  = new URL(servletURL);
      //System.out.println("url formed");
  
      
      //HttpURLConnection uc = (HttpURLConnection)u.openConnection();
      //System.out.println("http url connection opened ");
      //uc.setRequestMethod("Get");
      //System.out.println("request method set to GET");
      //uc.setDoOutput(true);
      //System.out.println("set do output to true");
      //ObjectOutputStream objOut = new ObjectOutputStream(new GZIPOutputStream(uc.getOutputStream()));
      //System.out.println("Object output stream created");
      //objOut.writeObject(obj);
      //System.out.println("object written to object output stream.");
      //objOut.flush();
      //objOut.close();
      //System.out.println("object output stream flushed and closed.");
      //System.out.println("Object sent !");							
      
      
      ObjectInputStream objIn = new ObjectInputStream(new GZIPInputStream(u.openStream()));
      Object rtn = objIn.readObject();
      objIn.close();
      

      System.out.println("Object received !");

      return rtn;
  }
  */

}


/**
* An extension of the built-in random generator to supply random
* integers as well
*/
/*
class Random extends java.util.Random {
	Random(int seed) {
		super(seed);
	}
	*/
	/**
	* @return a random integer in the range 0 to n-1
	*/
/*
	int nextInt(int n) {
		return (int)(nextDouble() * n);
	}
}
*/

/**
* Just a data structure to hold the pointer to a subtree as well as to
* its parent. This is needed to splice trees.
*/
class TreeHook {
	TreeHook(Program subtree, Function parent, int childIndex) {
		this.subtree = subtree;
		this.parent = parent;
		this.childIndex = childIndex;
	}
	Program subtree;
	Function parent;
	int childIndex;
}


/**
* Just a helper class.
*/
/*
abstract class Condition {
	abstract boolean test(Program p);
}
*/

/**
*
*/
class IsProgram extends Condition {
	boolean test(Program p) { return (p instanceof Program); }
}

/**
*
*/
class IsFunction extends Condition {
	boolean test(Program p) { return (p instanceof Function); }
}


/**
*	This is the base class for all kinds of tree nodes.
*/
/*
abstract class Program implements Cloneable, Serializable {
*/
	/**
	* @param level the current recursion level when descending a subtree
	* @return the text representation of the subtree
	*/
/*
	public abstract String toString(int level);
	*/
	/**
	* @return a text description of the node. While toString() prints
	*		the value of a subtree, getName() returns something like the
	*		class name of this very node. getName() can be used to preset
	*		the choice lists for the function and terminal sets.
	*/
/*
	abstract String getName();
	*/
	/**
	* @return a series of blanks proportional to the recursion level
	*/
/*
	public String indent(int level) {
		String s = new String();
		for (int i = 0; i < level; i++) {
			s = s + "  ";
		}
		return s;
	}
	*/
	/**
	* @return the value of the subtree, evaluated at x, i.e. y = f(x)
	*/
/*
	abstract double eval(double x);
	*/
	/**
	* @return the number of nodes in this subtree
	*/
/*
	abstract int countNodes();
	*/
	/**
	* @return the number of nodes in this subtree satisfying a certain
	*		condition
	*/
/*
	abstract int countNodes(Condition cond);
	*/
	/**
	* @return a deep copy of the subtree
	*/
/*
	abstract protected Object clone();
}
*/

/**
*	This is the base class for all kinds of terminal nodes.
*/
/*
abstract class Terminal extends Program implements Serializable {

	int countNodes() {
		return 1;
	}

	int countNodes(Condition cond) {
		return (cond.test(this)) ? 1 : 0;
	}

}

*/
/**
*
*/
/*
class RandomConstant extends Terminal implements Serializable {
	static final double MIN = -4.0;
	static final double MAX = +4.0;
	double value;

	RandomConstant() {
		value = GP.random.nextDouble() * (MAX - MIN) + MIN;
	}

	private RandomConstant(double value) {
		this.value = value;
	}

	public String toString(int level) {
		return indent(level) + value;
	}

	protected Object clone() {
		return new RandomConstant(this.value); };

	String getName() {
		return "Random Constant";
	}

	double eval(double x) {
		return value;
	}

}

*/
/**
*
*/
/*
class Variable extends Terminal implements Serializable {

	public String toString(int level) {
		return indent(level) + "x";
	}

	protected Object clone() {
		return new Variable();
	}

	String getName() {
		return "x";
	}

	double eval(double x) {
		return x;
	}

}

*/
/**
* This is the base class for all kinds of function nodes (i.e.
* non-terminals). Each kind of function can have a different number
* of arguments (i.e. subtrees).
*/
/*
abstract class Function extends Program implements Serializable {

	protected Program[] arg;

	public String toString(int level) {
		//	Just to please the eye: If all arguments of the function
		//	are terminals, I print a single line instead of spreading the 
		//	stuff tree-like.
		boolean allArgsAreTerminals = true;
		for (int a = 0; a < arg.length; a++) {
			allArgsAreTerminals = allArgsAreTerminals && (arg[a] instanceof Terminal);
		}
		String s = new String();
		if (!allArgsAreTerminals) {
			s = indent(level) + getName() + "(\n";
			int i = 0;
			while (i < arg.length-1) {
				s = s + arg[i].toString(level+1) + ",\n";
				i++;
			}
			if (i < arg.length) {
				s = s + arg[i].toString(level+1) + "\n";
			}
			s = s + indent(level) + ")";
		} else {
			s = indent(level) + getName() + "(";
			int i = 0;
			while (i < arg.length-1) {
				s = s + arg[i].toString(0) + ",";
				i++;
			}
			if (i < arg.length) {
				s = s + arg[i].toString(0);
			}
			s = s + ")";
		}
		return s;
	}

	protected Object clone() {
		Function temp = null;
		try {
			temp = (Function)getClass().newInstance();
			for (int i = 0; i < arg.length; i++) {
				temp.arg[i] = (Program)arg[i].clone();
			}
		} catch (Exception e) {
		}
		return temp;
	}

	int countNodes() {
		int count = 1;
		for (int a = 0; a < arg.length; a++) {
			count = count + arg[a].countNodes();
		}
		return count;
	}

	int countNodes(Condition cond) {
		int count = (cond.test(this) ? 1 : 0);
		for (int a = 0; a < arg.length; a++) {
			count = count + arg[a].countNodes(cond);
		}
		return count;
	}

}
*/
/*
class Addition extends Function implements Serializable {

	Addition() {
		arg = new Program[2];
	}

	String getName() {
		return "add";
	}

	double eval(double x) {
		return arg[0].eval(x) + arg[1].eval(x);
	}

}


class Subtraction extends Function implements Serializable {

	Subtraction() {
		arg = new Program[2];
	}

	String getName() {
		return "sub";
	}

	double eval(double x) {
		return arg[0].eval(x) - arg[1].eval(x);
	}

}


class Multiplication extends Function implements Serializable {

	Multiplication() {
		arg = new Program[2];
	}

	String getName() {
		return "mul";
	}

	double eval(double x) {
		return arg[0].eval(x) * arg[1].eval(x);
	}

}


class Division extends Function implements Serializable {

	Division() {
		arg = new Program[2];
	}

	String getName() {
		return "div";
	}

	double eval(double x) {
		double divisor = arg[1].eval(x);
		return (divisor == 0 ? 1 : arg[0].eval(x) / divisor);
	}

}


class Sine extends Function implements Serializable {

	Sine() {
		arg = new Program[1];
	}

	String getName() {
		return "sin";
	}

	double eval(double x) {
		return Math.sin(arg[0].eval(x));
	}

}


class Cosine extends Function implements Serializable {

	Cosine() {
		arg = new Program[1];
	}

	String getName() {
		return "cos";
	}

	double eval(double x) {
		return Math.cos(arg[0].eval(x));
	}

}


class Exp extends Function implements Serializable {

	Exp() {
		arg = new Program[1];
	}

	String getName() {
		return "exp";
	}

	double eval(double x) {
		double v = arg[0].eval(x);
		if (v > 100) {
			v = 100;
		} else if (v < -100) {
			v = -100;
		}
		return Math.exp(v);
	}

}


class First extends Function implements Serializable {

	First() {
		arg = new Program[2];
	}

	String getName() {
		return "first";
	}

	double eval(double x) {
		return arg[0].eval(x);
	}

}


class Second extends Function implements Serializable {

	Second() {
		arg = new Program[2];
	}

	String getName() {
		return "second";
	}

	double eval(double x) {
		return arg[1].eval(x);
	}

}
*/

/**
* This class combines a program and its computed fitness values.
*/
/*
class Individual implements Serializable {
	public Program program;
	double standardizedFitness;
	double adjustedFitness;
	double normalizedFitness;
	int hits;
        boolean sent;

	Individual(Program p) {
		program = (Program)p.clone();
		standardizedFitness = 0.0;
		adjustedFitness = 0.0;
		normalizedFitness = 0.0;
		hits = 0;
		sent = false;
	}

	Individual copy() {
		Individual newInd = new Individual(this.program);
		newInd.standardizedFitness = this.standardizedFitness;
		newInd.adjustedFitness = this.adjustedFitness;
		newInd.normalizedFitness = this.normalizedFitness;
		newInd.hits = this.hits;
		newInd.sent = this.sent;
		return newInd;
	}

}
*/

/**
* Just a helper class: it combines the class template and its
* text description of a node. It is used to build the choice lists
* for function and terminal sets.
*/
/*
class ProgramChoice {
	private Class cls;
	private String text;

	ProgramChoice(String text, Class cls) {
		this.cls = cls;
		this.text = text;
	}

	public String toString() { return text; }
	Class value() { return cls; }
}
*/

/**
* Just a helper class: it combines the enumeration value and its
* text description of a generation method. It is used to build the
* choice list for the generation method menu.
*/
class MethodOfGeneration implements Serializable {
	static final int GROW = 0;
	static final int FULL = 1;
	static final int RAMPED_HALF_AND_HALF = 2;

	private String text;
	private int value;

	MethodOfGeneration(String text, int value) {
		this.text = text;
		this.value = value;
	}

	int getValue() {
		return this.value;
	}

	public String toString() {
		return this.text;
	}
}


/**
* Just a helper class: it combines the enumeration value and its
* text description of a selection method. It is used to build the
* choice list for the selection method menu.
*/
class MethodOfSelection implements Serializable {
	static final int FITNESS_PROPORTIONATE = 0;
	static final int TOURNAMENT = 1;

	private String text;
	private int value;

	MethodOfSelection(String text, int value) {
		this.text = text;
		this.value = value;
	}

	public int getValue() {
		return this.value;
	}

	public String toString() {
		return this.text;
	}
}


/**
* Just a helper class: it combines the enumeration value and its
* text description of whether elitism is used. It is used to build the
* choice list for the elitism menu.
*/
/*
class ElitismSelection implements Serializable {
	static final int ELITISM_USED = 0;
	static final int ELITISM_NOT_USED = 1;

	private String text;
	private int value;

	ElitismSelection(String text, int value) {
		this.text = text;
		this.value = value;
	}

	public int getValue() {
		return this.value;
	}

	public String toString() {
		return this.text;
	}
	
}
*/

