package bham.cs.fsc.gp.ant;

import java.io.*;
import bham.cs.fsc.gp.*;
import bham.cs.fsc.ui.*;
//import gp.ant.ui.*;
//import gp.approx.ui.*;

/**
* Problem specific code to demonstrate the solving of the Artificial Ant following the Los Altos Hills problem.
*
* @version 1.0
* @author Phyllis Chong (<a href="mailto:fsc@cs.bham.ac.uk">fsc@cs.bham.ac.uk</a>)
*/

public class LosAltosHills extends Ant {
  public PlotUI fitnessCases;
  public String fitnessCaseString;

  final int numFoodPieces = 157;
  final int gridSize = 100;  
  final String panelName = "Los Altos Hills Trail";
  final int probTimeSteps = 3000;


  public LosAltosHills() { 
    Ant.numFoodPieces = this.numFoodPieces;
    Ant.gridSize = this.gridSize;
    Ant.panelName = this.panelName;
    Ant.probTimeSteps = this.probTimeSteps;
}

  public String getProblemName() {
    return "Artificial Ant following the Los Altos Hills Trail"; 
  }

  public SetUI getFunctionSet() {

		ProgramChoice[] functionChoices = {
			new ProgramChoice("IF-FOOD-AHEAD", new IfFoodAhead().getClass()),
			new ProgramChoice("PROGN2", new Progn2().getClass()),
			new ProgramChoice("PROGN3", new Progn3().getClass()),
			new ProgramChoice("PROGN4", new Progn4().getClass())
		};
		int[] functionSelections = {0, 1, 2, 3};
		SetUI functionSet = new SetUI("Function set", functionChoices, functionSelections);
		
		return functionSet;
		
  }

  void placeFoodPieces() {

    food[1][0] = true;
    food[2][0] = true;
    food[3][0] = true;
    food[3][1] = true;
    food[3][2] = true;
    food[3][3] = true;
    food[3][4] = true;
    food[3][5] = true;
    food[4][5] = true;
    food[5][5] = true;
    food[6][5] = true;
    food[8][5] = true;
    food[9][5] = true;
    food[10][5] = true;
    food[11][5] = true;
    food[12][5] = true;
    food[12][6] = true;
    food[12][7] = true;
    food[12][8] = true;
    food[12][9] = true;
    food[12][11] = true;
    food[12][12] = true;
    food[12][13] = true;
    food[12][14] = true;
    food[12][17] = true;
    food[12][18] = true;
    food[12][19] = true;
    food[12][20] = true;
    food[12][21] = true;
    food[12][22] = true;
    food[12][23] = true;
    food[11][24] = true;
    food[10][24] = true;
    food[9][24] = true;
    food[8][24] = true;
    food[7][24] = true;
    food[4][24] = true;
    food[3][24] = true;
    food[1][25] = true;
    food[1][26] = true;
    food[1][27] = true;
    food[1][28] = true;
    food[1][29] = true;
    food[1][30] = true;
    food[1][31] = true;
    food[2][33] = true;
    food[3][33] = true;
    food[4][33] = true;
    food[5][33] = true;
    food[7][32] = true;
    food[7][31] = true;
    food[8][30] = true;
    food[9][30] = true;
    food[10][30] = true;
    food[11][30] = true;
    food[12][30] = true;
    food[17][30] = true;
    food[18][30] = true;
    food[20][29] = true;
    food[20][28] = true;
    food[20][27] = true;
    food[20][26] = true;
    food[20][25] = true;
    food[20][24] = true;
    food[20][21] = true;
    food[20][20] = true;
    food[20][19] = true;
    food[20][18] = true;
    food[21][15] = true;
    food[24][14] = true;
    food[24][13] = true;
    food[24][10] = true;
    food[24][9] = true;
    food[24][8] = true;
    food[24][7] = true;
    food[25][5] = true;
    food[26][5] = true;
    food[27][5] = true;
    food[28][5] = true;
    food[29][5] = true;
    food[30][5] = true;
    food[31][5] = true;
    food[32][5] = true;
    food[33][5] = true;
    food[34][5] = true;
    food[35][5] = true;
    food[36][5] = true;
    food[38][4] = true;
    food[38][3] = true;
    food[39][2] = true;
    food[40][2] = true;
    food[41][2] = true;
    food[43][3] = true;
    food[43][4] = true;
    food[43][6] = true;
    food[43][9] = true;
    food[43][12] = true;
    food[42][14] = true;
    food[41][14] = true;
    food[40][14] = true;
    food[37][15] = true;
    food[38][18] = true;
    food[41][19] = true;
    food[40][22] = true;
    food[37][23] = true;
    food[37][24] = true;
    food[37][25] = true;
    food[37][26] = true;
    food[37][27] = true;
    food[37][28] = true;
    food[37][29] = true;
    food[37][30] = true;
    food[37][31] = true;
    food[37][32] = true;
    food[37][33] = true;
    food[37][34] = true;
    food[35][34] = true;
    food[35][35] = true;
    food[35][36] = true;  
    food[35][37] = true;
    food[35][38] = true;
    food[35][39] = true;
    food[35][40] = true;
    food[35][41] = true;
    food[35][42] = true;
    food[34][42] = true;
    food[33][42] = true;
    food[32][42] = true;
    food[31][42] = true;
    food[30][42] = true;
    food[30][44] = true;
    food[30][45] = true;
    food[30][46] = true;
    food[30][47] = true;
    food[30][48] = true;
    food[30][49] = true;
    food[28][50] = true;
    food[28][51] = true;
    food[28][52] = true;
    food[28][53] = true;
    food[28][54] = true;
    food[28][55] = true;
    food[28][56] = true;
    food[27][56] = true;
    food[26][56] = true;
    food[25][56] = true;
    food[24][56] = true;
    food[23][56] = true;
    food[22][58] = true;
    food[22][59] = true;
    food[22][60] = true;
    food[22][61] = true;
    food[22][62] = true;
    food[22][63] = true;
    food[22][64] = true;
    food[22][65] = true;
    food[22][66] = true;
    

    // Print grid for verification
    /*
     for (int i=0; i<32; i++) {
      for (int j=0; j<32; j++) {
	System.out.println("Food at " + i + j + " is " + Ant.food[i][j]);
      }
     }
     */
  } 
       
  /*  
public Histogram createHistogram() {
  //System.out.println("Setting window for Los Altos Hills");
  return new Histogram(0.0, 0.0, 1.0, 157.0);
}
*/

}


















