package bham.cs.fsc.gp.approx;

//import java.util.Observable;
//import java.util.Vector;
//import java.util.Hashtable;
//import java.util.Enumeration;
//import java.rmi.*;
//import java.net.*;
import java.io.*;
import java.awt.*;
//import java.awt.Panel;
import bham.cs.fsc.gp.*;
import bham.cs.fsc.ui.*;
import bham.cs.fsc.exp.*;


/**
* Problem specific code to demonstrate the solving of the Symbolic Regression problem.
*
* @version 1.0
* @author Phyllis Chong (<a href="mailto:fsc@cs.bham.ac.uk">fsc@cs.bham.ac.uk</a>)
*/

public class Approx implements Problem {

  //public XYPlot plot;
  public UPlot plot;
  //public PointsUI fitnessCases;
  public PlotUI fitnessCases;
  public String fitnessCaseString;
  //public int NrOfFitnessCases = 20;
  public int NrOfFitnessCases = 10;
  public boolean maxHits = false;


  public Approx() { }


  public String getProblemName() {
    return "Approximation (Symbolic Regression)";
  }

  public Panel defPlotPanel(DGPClient dgpc) {
    Panel plotPanel = new Panel();
    plotPanel.setLayout(new BorderLayout());
    //String probLabel = prob.getGraphicsLabel();
    //plotPanel.add("North", new Label(probLabel));
    plotPanel.add("North", new Label("Approximation"));
		
    //plot = new XYPlot();
    plot = new UPlot();
    //plot = prob.getGraphics();	
    plotPanel.add("Center", plot);
    
    return plotPanel;
  }

  /*
  public UPlot getPlot() {
    return plot;
  }
  */

  public Plot getPlot() {
    Plot pplot = plot;
    return pplot;
  }

  /*
  public String getGraphicsLabel() {
    return "Approximation";
  }

  public XYPlot getGraphics() {
    XYPlot graphics = new XYPlot();
    return graphics;
  }
  */

  public SetUI getTerminalSet() {

		ProgramChoice[] terminalChoices = {
			new ProgramChoice("Random constant", new RandomConstant().getClass()),
			new ProgramChoice("Variable x", new Variable().getClass())
		};
		int[] terminalSelections = {0, 1};
		SetUI termSet = new SetUI("Terminal set",	terminalChoices, terminalSelections);
		
		return termSet;
  }
		

  public SetUI getFunctionSet() {

		ProgramChoice[] functionChoices = {
			new ProgramChoice("+ (add)", new Addition().getClass()),
			new ProgramChoice("- (sub)", new Subtraction().getClass()),
			new ProgramChoice("* (mul)", new Multiplication().getClass()),
			new ProgramChoice("/ (div)", new Division().getClass()),
			new ProgramChoice("sin", new Sine().getClass()),
			new ProgramChoice("cos", new Cosine().getClass()),
			new ProgramChoice("exp", new Exp().getClass())
		};
		int[] functionSelections = {0, 1, 2, 3};
		//int[] functionSelections = {0, 1, 2, 3, 4, 5, 6};
		SetUI functionSet = new SetUI("Function set", functionChoices, functionSelections);
		
		return functionSet;
  }


  //public RealPoint[] fitnessFunction(Individual ind, GP gp, int individualNr) {
  //public void fitnessFunction(Individual ind, GP gp, int individualNr) {
  public boolean fitnessFunction(Individual ind, GP gp, int individualNr) {
       /**
	* This function has to evaluate the fitness for a specific individual.
	* As a side-effect, it notfies any observers of the GP that an individual
	* has been evaluated and supplies the test cases.
	*/
    
    gp.notifyObservers(new GPMessageStatus("Evaluating individual #" + individualNr + "\n" +
					"of generation " + gp.currentGeneration));
    
                boolean solutionFound = false ;
		double rawFitness = 0.0;
		ind.hits = 0;
		//RealPoint[] testCases = new RealPoint[gp.fitnessCases.get().length];
		//RealPoint[] testCases = new RealPoint[fitnessCases.get().length];
		PlotData[] testCases = new RealPoint[fitnessCases.get().length];
		//for (int index = 0; index < gp.fitnessCases.get().length; index++) {
		for (int index = 0; index < fitnessCases.get().length; index++) {  
			double x = fitnessCases.get()[index].x;
			double valueFromProgram = ind.program.eval(x);
			double difference = Math.abs(valueFromProgram - fitnessCases.get()[index].y);
			rawFitness = rawFitness + difference;
			if (difference < 0.01) {
				ind.hits++;
			}
			testCases[index] = new RealPoint(x, valueFromProgram);
		}

		ind.standardizedFitness = rawFitness;

		if (ind.hits == NrOfFitnessCases) { 
		  maxHits = true;
		  //sent variable set to true so that the solution will only be sent at end of run and not at exportIndividual.
		  ind.sent = true;
		  solutionFound = true;
		}

		//gp.notifyObservers(new GPMessageEvaluatingIndividual(gp.currentGeneration, individualNr, testCases));
		gp.notifyObservers(new GPMessageEvaluatingIndividual(testCases));
		
		return solutionFound;	
  }


  public void evalBestOfRunIndividual(Individual bestIndv, GP gp) {
        /**
	* This function has to evaluate the fitness for a specific individual.
	* As a side-effect, it notfies any observers of the GP that an individual
	* has been evaluated and supplies the test cases.
	*/
        
                //RealPoint[] testCases = new RealPoint[fitnessCases.get().length];
		PlotData[] testCases = new RealPoint[fitnessCases.get().length];
		for (int index = 0; index < fitnessCases.get().length; index++) {
			double x = fitnessCases.get()[index].x;
			double y = bestIndv.program.eval(x);
			
			testCases[index] = new RealPoint(x, y);
		}
		//return testCases;

		gp.notifyObservers(new GPMessageBestFound(gp.currentGeneration,
					bestIndv.program.toString(0),
					testCases,
					bestIndv.adjustedFitness));
		
  }


/**
	* Computes the normalized and adjusted fitness of each
	* individual in the population.
	*/
        
	public void normalizeFitnessOfPopulation(Individual[] population) {
		double sumOfAdjustedFitnesses = 0.0;
		double avgAdjFitness = 0.0;

		for (int i = 0; i < population.length; i++) {
			//	Set the adjusted fitness.
			population[i].adjustedFitness = 
    			1.0 / (population[i].standardizedFitness + 1.0);
			//	Add up the adjusted fitnesses so that we can normalize them.
			sumOfAdjustedFitnesses =
					sumOfAdjustedFitnesses + population[i].adjustedFitness;
		}
		//	Loop through population normalizing the adjusted fitness.
		for (int i = 0; i < population.length; i++) {
			population[i].normalizedFitness =
					population[i].adjustedFitness / sumOfAdjustedFitnesses;
		}
		avgAdjFitness = sumOfAdjustedFitnesses / population.length;
		System.out.println("\nAverage adjusted fitness of population for generation is : "  + avgAdjFitness);


	}
	


  public double[] getFitnessesOfAllIndividuals(Individual[] population) {
    double[] fitness = new double[population.length];
    for (int i = 0; i < fitness.length; i++) {
      fitness[i] = population[i].adjustedFitness;  
    }   
    return fitness;
  }
	
  //public String makeDefaultFitnessCases() {
  public void makeDefaultFitnessCases() {
                //NrOfFitnessCases = 20;
                //String s = "//3*x^4 - 3*x + 1\n";
                //String s = "//Fitness function : 0.2 + x * exp(x) -3 * x^3 / cos(cos(exp(x) + x -1))\n";
                //String s = "//Fitness function : x^4 + x^3 + x^2 + x\n";
                String s = "//Fitness function : 0.5 * x^2\n";
		for (int i = 0; i < NrOfFitnessCases; i++) {
		        double x = (double)i / (NrOfFitnessCases - 1);
		        //int x = random.nextInt(100); 

			if (x == 0.0) {
				x = 1.0e-7;
			}

			//double y = 1.0 + (-3.0)*x + (3.0)*x*x*x*x;
			//double y = 1.0 + x*x*x* + (- x*x);
			//double y = x*x + (-1.8)*x + 0.7;
			//double y = Math.cos(Math.cos(x*x + x + (-1.0)));
			//double y = x*x + (-1.8)*x + 0.7 / Math.cos(Math.cos(x*x + x + (-1.0)));
			//*The following equation is a difficult one.
			//double y = 0.2 + x * Math.exp(x) + (-3.0)*x*x*x / Math.cos(Math.cos(Math.exp(x) + x + (-1.0)));
			//*The following equation is mentioned in Koza's book
			//double y = x*x*x*x + x*x*x + x*x + x;
			//*The following equation is mentioned in appendix B of Koza's book & used in the experiment here.
			double y = (0.5)*x*x;
			//double y = (5.0)*x*x*x + (- x*x*x*x) + (2.0);

			//	The stream tokenizer does not handle numbers with an exponent.
			//	Therefore, clamp small numbers like 1.2e-6 to 0: 
			if (Math.abs(x) < 1.0e-5) {
				x = 0;
			};
			if (Math.abs(y) < 1.0e-5) {
				y = 0;
			};
			s = s + x + "  " + y + "\n";
		}
		//return s;
		fitnessCaseString = s;
		fitnessCases = new PointsUI("Fitness cases", s);

  }

public PlotUI getFitnessCases() {
  return fitnessCases;
}

public String getFitnessCaseString() {
  return fitnessCaseString;
}

public int getNrOfFitnessCases() {
  return NrOfFitnessCases;
}

public void initMaxHits() {
  maxHits = false;
}

public boolean getMaxHits() {
  return maxHits;
}

public String getFitnessType() {
  return "adjusted fitness";
}


  /*
public void setPWindow() {
  System.out.println("??? setting window for Approx");
  new Plot().setWindow(0.0, 0.0, 1.0, 1.0);
}
*/

public Histogram createHistogram() {
  //System.out.println("??? setting window for Approx");
  return new Histogram(0.0, 0.0, 1.0, 1.0);
}


public FrameConstraint getPlotFrameConstraint() {
  return new FrameConstraint(
			     0.0, 4,
			     0.0, 0,
			     0.5, -4,
			     0.5, 0
			     ) ;
}

public FrameConstraint getHistoFrameConstraint() {
  return new FrameConstraint(
			     0.0, 4,
			     0.5, 0,
			     0.5, -4,
			     1.0, 0
			     ) ;
}

public Image[] getImageArray() {return null;}
  


        /**
	* @returns <code>true</code> if the GP algorithm should terminate
	*/
        /*
	public boolean terminationPredicate() {
	  //return currentGeneration >= MAX_GENERATIONS;
	  //return currentGeneration >= maxGenNum.intValue();
	  if (currentGeneration >= maxGenNum.intValue() || maxHits) {
	    return true;
	  }
	}
	*/

}










